// Fun routines for manipulating alpha-blended sprites.
// Of course, this silliness wouldn't be necessary if Allegro just included these!
// Matt Sarnoff, Oct. 26, 2004.
// 10/28/04: updated to support 16-bit color.
// 10/29/04: *_alpha_* functions now work properly in 16-bit color.

// Blender functions for modifying an alpha sprite's overall transparency
// Thanks to smart_idiot on gamedev.net, and Fladimir da Gorf on allegro.cc
// for the 16-bit version. Trivial-rejection condition added by yours truly.

#include "alphasprites.h"

unsigned long custom_blender32(unsigned long x,unsigned long y, unsigned long n)
{
  if (geta32(x) == 0)
    return y;
  
  unsigned long v = (geta32(x)*n)>>8, i = (~v)&0xff;
  return makecol32((getr32(x)*v+getr32(y)*i)>>8,
                   (getg32(x)*v+getg32(y)*i)>>8,
                   (getb32(x)*v+getb32(y)*i)>>8);
}

unsigned long custom_blender16(unsigned long x,unsigned long y, unsigned long n)
{
  if (geta32(x) == 0)
    return y;

  unsigned long v = (geta32(x)*n)>>8, i = (~v)&0xff;
  return makecol16((getr32(x)*v+getr16(y)*i)>>8,
                   (getg32(x)*v+getg16(y)*i)>>8,
                   (getb32(x)*v+getb16(y)*i)>>8);
}


// Draws an alpha-blended sprite with overall transparency alpha.
void alpha_trans_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int alpha)
{
  TRANS_ALPHA_BLENDER;
  draw_trans_sprite(bmp, sprite, x, y); 
}  

// Draws an alpha-blended sprite stretched to the specified size.
void stretch_trans_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int w, int h)
{
  BITMAP *temp = create_bitmap_ex(32,w, h);
  clear_bitmap(temp);
  stretch_blit(sprite, temp, 0, 0, sprite->w, sprite->h, 0, 0, w, h);
  draw_trans_sprite(bmp, temp, x, y);
  destroy_bitmap(temp);
}

// Like stretch_trans_sprite, except it allows you to specify the overall transparency.
void stretch_alpha_trans_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int w, int h, int alpha)
{
  TRANS_ALPHA_BLENDER;
  stretch_trans_sprite(bmp, sprite, x, y, w, h); 
}

// Rotates an alpha-blended sprite by the specified angle (as a fixed).
// **** Do not, under any circumstances, use the rotate_* functions.
// The stretch functions should be ok, but these are SLOW AS SHIT. You've been warned.
void rotate_trans_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, fixed angle)
{
  int diagonal = (int)sqrt((float)(sprite->w*sprite->w + sprite->h*sprite->h));
  BITMAP *temp = create_bitmap_ex(32, diagonal, diagonal);
  clear_bitmap(temp);
  rotate_sprite(temp, sprite, (diagonal-sprite->w)>>1, (diagonal-sprite->h)>>1, angle);
  draw_trans_sprite(bmp, temp, x, y);
  destroy_bitmap(temp);
}

// Rotates an alpha-blended sprite by the specified angle, with user-specified overall
// transparency.
void rotate_alpha_trans_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, fixed angle, int alpha)
{
  TRANS_ALPHA_BLENDER;
  rotate_trans_sprite(bmp, sprite, x, y, angle); 
}

// Now things start to get crazy. Stretches an alpha-blended sprite to the specified
// size, then rotates it by the specified angle.
void rotate_stretch_trans_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int w, int h, fixed angle)
{
  BITMAP *temp = create_bitmap_ex(32, w, h);
  clear_bitmap(temp);
  stretch_blit(sprite, temp, 0, 0, sprite->w, sprite->h, 0, 0, w, h);
  rotate_trans_sprite(bmp, temp, x, y, angle);
  
  destroy_bitmap(temp);
}

// Combines all three. Stretches an alpha-blended sprite to the specified
// size, then rotates it by the specified angle. Oh yeah, did we mention you can
// specify the overall transparency as well? Quite the hack.
void rotate_stretch_alpha_trans_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int w, int h, fixed angle, int alpha)
{
  TRANS_ALPHA_BLENDER;
  rotate_stretch_trans_sprite(bmp, sprite, x, y, w, h, angle);
}  
