#include "alphatrail.h"
#include "saturn.h"

int polyTempPoints[8];
// Uses Allegro's polygon() routine to draw a quadrilateral.
void quadrilateral(BITMAP *bmp, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int color)
{
  polyTempPoints[0] = x1;   polyTempPoints[1] = y1;
  polyTempPoints[2] = x2;   polyTempPoints[3] = y2;
  polyTempPoints[4] = x3;   polyTempPoints[5] = y3;
  polyTempPoints[6] = x4;   polyTempPoints[7] = y4;
  polygon(bmp, 4, polyTempPoints, color);
}

// Linear interpolation, between points y0 and y1, at point mu [0, 1].
float lerp(float y0, float y1, float mu)
{ return y0+mu*(y1-y0); }

// Cubic interpolation, between points y1 and y2 at point mu [0, 1].
// Choose good values for y0 and y3 (y0 < y1 and y3 > y2) to get a nice curve.
float cuberp(float y0, float y1, float y2, float y3, float mu)
{
   float a0,a1,a2,a3,mu2;
   mu2 = mu*mu;
   
   // calculate coefficients
   a0 = y3 - y2 - y0 + y1;
   a1 = y0 - y1 - a0;
   a2 = y2 - y0;
   a3 = y1;
   return a0*mu*mu2 + a1*mu2 + a2*mu + a3;
}

alphatrail::alphatrail()
{ reset(); }

alphatrail::alphatrail(alphatrail &copyAlphaTrail) // copy constructor
    { // this is used when removing, since they're kept in a linear array
    count=copyAlphaTrail.count;
    for(int i=0;i<length;i++)
      {
      seg[i].prev=copyAlphaTrail.seg[i].prev;
      seg[i].x=copyAlphaTrail.seg[i].x;
      seg[i].y=copyAlphaTrail.seg[i].y;
      seg[i].xang=copyAlphaTrail.seg[i].xang;
      seg[i].yang=copyAlphaTrail.seg[i].yang;
      }
    headIndex=copyAlphaTrail.headIndex;
    tailUsed=copyAlphaTrail.tailUsed;
    headThickness=copyAlphaTrail.headThickness;
    tailThickness=copyAlphaTrail.tailThickness;
    length=copyAlphaTrail.length;
    color=copyAlphaTrail.color;
    }
    
void alphatrail::reset()
{
  count = headIndex = tailUsed = 0;

  length=TAIL_MAX_LENGTH;
  
  for (int i = 0; i < length; i++)
  { 
    seg[i].prev = -1;
    seg[i].x = seg[i].y = 0;
    seg[i].xang = seg[i].yang = 0;
  }
}
    
void alphatrail::set(int newHeadThickness, int newTailThickness, int newLength, int newColor)
{
  reset();
  headThickness = newHeadThickness;
  tailThickness = newTailThickness;
  if(length>=TAIL_MAX_LENGTH)
    length=TAIL_MAX_LENGTH-1;
  if(length<=0)
    length=1;
  length = newLength;
  color = newColor;
}
    
void alphatrail::addSegment(int newx, int newy, int angle)
{
  int prev = headIndex;
  headIndex++;
  
  if (tailUsed < length-1)
    tailUsed++;
  if (headIndex > tailUsed && count > 0)
    headIndex = 0;

  seg[headIndex].prev = prev;
  seg[headIndex].x = newx;
  seg[headIndex].y = newy;
  seg[headIndex].xang = cos(angle);
  seg[headIndex].yang = sin(angle);
    
  count++;
}
    
void alphatrail::draw()
{
  drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
  int c = headIndex;
  for (int i = tailUsed; i > 1; i--) // kluge
  {
    float pos = (float)i/length, prevPos = (float)(i-1)/length;

    // Linear interpolation, or cubic interpolation?
    // (for cubic, the "0" and the "15" are extra coefficients that affect the curviness  
    //int th = lerp(tailThickness, headThickness, pos);
    //int th2 = lerp(tailThickness, headThickness, (float)(i-1)/TAIL_MAX_LENGTH);    
    int th = int(cuberp(0, tailThickness, headThickness, 15, pos));
    int th2 = int(cuberp(0, tailThickness, headThickness, 15, prevPos));
     
    // Original drawing mode, using Allegro's polygon() function to draw filled trapezoids,
    // but with seams between segments
    set_add_blender(255, 255, 255, (int)(pos*255));
    quadrilateral(DB, int(seg[c].x-th*seg[c].xang)-viewport.x, int(seg[c].y-th*seg[c].yang)-viewport.y,
                      int(seg[seg[c].prev].x-th2*seg[seg[c].prev].xang)-viewport.x, int(seg[seg[c].prev].y-th2*seg[seg[c].prev].yang)-viewport.y,
                      int(seg[seg[c].prev].x+th2*seg[seg[c].prev].xang)-viewport.x, int(seg[seg[c].prev].y+th2*seg[seg[c].prev].yang)-viewport.y,
                      int(seg[c].x+th*seg[c].xang)-viewport.x, int(seg[c].y+th*seg[c].yang)-viewport.y,
                      makecol(255, 128, 0));
    
    // New drawing code, drawing only line segments. Seams are gone, but additive blending is
    // now impossible because many of the lines overlap each other, ruining the effect.
    // For some reason, the color instance variable isn't working right...had to hardcode a color here.
    /*set_trans_blender(255, 255, 255, (int)(pos*255));
    for (float p = 0; p < 1; p += 0.01)
    {
      int x1 = lerp(seg[c].x-th*seg[c].xang, seg[seg[c].prev].x-th2*seg[seg[c].prev].xang, p);
      int y1 = lerp(seg[c].y-th*seg[c].yang, seg[seg[c].prev].y-th2*seg[seg[c].prev].yang, p);
      int x2 = lerp(seg[c].x+th*seg[c].xang, seg[seg[c].prev].x+th2*seg[seg[c].prev].xang, p);
      int y2 = lerp(seg[c].y+th*seg[c].yang, seg[seg[c].prev].y+th2*seg[seg[c].prev].yang, p);
      line(DB, x1-viewport.x, y1-viewport.y, x2-viewport.x, y2-viewport.y, makecol(255, 128, 0));
    }*/
    
    c = seg[c].prev;
  }  
  drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
}
