#include "saturn.h"

void loadBombGraphics()
  {
  for(int type=0;type<BOMB_TYPES;type++)
    bombBreed[type].loadGraphics(type);
  }

void unloadBombGraphics()
  {
  for(int type=0;type<BOMB_TYPES;type++)
    bombBreed[type].unloadGraphics();
  }

bomb_breed_typ bombBreed[BOMB_TYPES];

  bomb_breed_typ::bomb_breed_typ()
    {
    myID=0;
    BOMB=NULL;
    damage=0;
    cost=150;
    shakeMin=15;
    shakeRandAddTo=15;
    dropBombReloadTime=20;
    capacityPerRun=5;
    stallOverallSpeed=2.4;
    noseDriftForce=0.2;
    }

  void bomb_breed_typ::loadGraphics(int myType)
    {
    BITMAP *scrap;
    PALETTE palette;

    scrap = load_pcx("data//bomb.pcx",palette);

    myID=myType;
    BOMB=create_bitmap(BOMB_IMAGEFILE_MAX_W,BOMB_IMAGEFILE_MAX_H);
    blit(scrap,BOMB,myID*BOMB_IMAGEFILE_MAX_W,0,0,0,
       BOMB_IMAGEFILE_MAX_W,BOMB_IMAGEFILE_MAX_H);

    switch(myType)
      {
      case BOMB_DUMB:
        cost=0;
        damage=10;
        shakeMin=15;
        shakeRandAddTo=15;
        dropBombReloadTime=15;
        capacityPerRun=1;
        stallOverallSpeed=2.4;
        noseDriftForce=0.2;
        break;
      case BOMB_HARPOON:
        cost=80;
        damage=5;
        shakeMin=8;
        shakeRandAddTo=7;
        dropBombReloadTime=8;
        capacityPerRun=1;
        stallOverallSpeed=HARPOON_BOMB_SPEED;
        noseDriftForce=0.0;
        break;
      case BOMB_CLUSTER:
        cost=280;
        damage=3;
        shakeMin=20;
        shakeRandAddTo=20;
        dropBombReloadTime=25;
        capacityPerRun=4;
        stallOverallSpeed=2.4;
        noseDriftForce=0.2;
        break;
      case BOMB_HE:
        cost=630;
        damage=42;
        shakeMin=20;
        shakeRandAddTo=20;
        dropBombReloadTime=25;
        capacityPerRun=3;
        stallOverallSpeed=2.4;
        noseDriftForce=0.2;
        break;
      case BOMB_BUNKERBUSTER:
        cost=750;
        damage=70;
        shakeMin=35;
        shakeRandAddTo=20;
        dropBombReloadTime=30;
        capacityPerRun=3;
        stallOverallSpeed=4.0;
        noseDriftForce=0.1;
        break;
      case BOMB_DOUBLE:
        cost=425;
        damage=14;
        shakeMin=20;
        shakeRandAddTo=20;
        dropBombReloadTime=15;
        capacityPerRun=1;
        stallOverallSpeed=2.4;
        noseDriftForce=0.2;
        break;
      case BOMB_EMP:
        cost=600;
        damage=4;
        shakeMin=5;
        shakeRandAddTo=5;
        dropBombReloadTime=10;
        capacityPerRun=5;
        stallOverallSpeed=2.4;
        noseDriftForce=0.2;
        break;
      case BOMB_PLASTIC:
        cost=375;
        damage=15;
        shakeMin=15;
        shakeRandAddTo=15;
        dropBombReloadTime=20;
        capacityPerRun=1;
        stallOverallSpeed=2.4;
        noseDriftForce=0.2;
        break; 
      case BOMB_LASER:
        cost=1200;
        damage=80;
        shakeMin=15;
        shakeRandAddTo=15;
        dropBombReloadTime=20;
        capacityPerRun=2;
        stallOverallSpeed=LASER_BOMB_SPEED;
        noseDriftForce=0.1;
        break;
      case BOMB_NAPALM:
        cost=1400;
        damage=8;
        shakeMin=10;
        shakeRandAddTo=5;
        dropBombReloadTime=10;
        capacityPerRun=3;
        stallOverallSpeed=2.4;
        noseDriftForce=0.2;
        break;
      case BOMB_NUKE:
        cost=20000;
        damage=10000;
        shakeMin=60;
        shakeRandAddTo=30;
        dropBombReloadTime=45;
        capacityPerRun=8;
        stallOverallSpeed=2.4;
        noseDriftForce=0.2;
        break;
      }

    destroy_bitmap(scrap);
    }

  void bomb_breed_typ::unloadGraphics()
    {
    destroy_bitmap(BOMB);
    }

bomb_typ bomb[NUM_LIVEBOMBS];

/* The actual class prototype is defined inside init.h */
  bomb_typ::bomb_typ()
    {

    }

  void bomb_typ::reset()
    {
    detonateOrVanishFlag=KEEPALIVE;
    }

  void bomb_typ::launch(int new_x, int new_y, int init_xv,int init_yv,int new_ang,int new_facing,int new_type)
    {
    if(new_type==BOMB_DOUBLE)
      {
      new_type=BOMB_DUMB;
      if(g_liveBombs<NUM_LIVEBOMBS-2) // enough room for extra?
        bomb[g_liveBombs+1].launch(new_x-new_facing*(4+(rand()%6)), new_y+(2+(rand()%5)),
               init_xv+new_facing*(2+(rand()%5)),init_yv+(2+(rand()%5)),
               new_ang+(rand()%30)-(rand()%30),new_facing,new_type);
      }

    g_liveBombs++;

    switch(new_type)
      {
      case BOMB_HARPOON:
        bombSpeed=HARPOON_BOMB_SPEED;
        target_x=-1;
        target_y=-1;
        break;
      case BOMB_LASER:
        bombSpeed=LASER_BOMB_SPEED; // 1 of 2 bombs that use this variable
        target_x=new_x;
        target_y=level.i_platformDriftHeightCurrent;
        target_timer=LASER_BOMB_LOCKON_TIMER;
        break;
      case BOMB_CLUSTER:
        target_x=-1;
        target_y=-1;
        target_timer=CLUSTER_BOMB_TIMER;
        break;
      default:
        target_x=target_y=-1;
        break;
      }
    detonateOrVanishFlag=KEEPALIVE;
    x=(float)(new_x);
    y=(float)(new_y);
    xv=(float)init_xv;
    yv=(float)init_yv;
    type=new_type;
    facing=new_facing;
    bombAngle=new_ang;
    }

  void bomb_typ::moveBomb()
    {
    float planeSpeedExtra;
    float bombSpeedExtra;
    float speedAsPerc;

    if(facing==FACING_RIGHT)
      {
      flippedFacingAngle=bombAngle;
      }
    else
      {
      flippedFacingAngle=255.0-(bombAngle-128.0);
      }

      // engine thrust

    if(type==BOMB_HARPOON)
      {
      if(rand()%6==1)
        {
        LAUNCH_EXPLOSION_PARTICLES_EX(1, x, y, 1.0, PARTICLE_SPARK);
        }
      xv=facing*bombSpeed*cos(bombAngle); // rocket support later?
      yv=bombSpeed*sin(bombAngle);
      }
    if(type==BOMB_LASER)
      {
      if(target_timer>0)
        target_timer--; // drift before acquiring lock
      else if(target_timer==0)
        {
        sound.playSound(SFXmenuClick,255,SCREEN_W>>1);
        target_timer--; // drift before acquiring lock
        facing=1; // make it face proper, since now it's largely angle driven
        }
      else
        {
        bombAngle=atan2(target_y-y,target_x-x);
        xv=bombSpeed*cos(bombAngle); // rocket support later?
        yv=bombSpeed*sin(bombAngle);
        }
      }
    else if(type==BOMB_CLUSTER)
      {
      target_timer--; // acts as bomb detonation airburst timer
      if(target_timer<=0 || y>level.i_platformDriftHeightCurrent-10)
        {
        detonateOrVanishFlag=DETONATE;
        sound.queueSound(SFXcrash,255,x);
        LAUNCH_EXPLOSION_PARTICLES_EX(5, x, y, 1.0, RANDOM_DEBRIS);
        return;
        }
      }
    bombSpeedExtra=PITCH_SPEED_AERODYN_EFFECT*sin(flippedFacingAngle);
    
    // facing down = faster fall
    /*xv*=((bombSpeedExtra+PITCH_SPEED_AERODYN_EFFECT*1.2)+10.0)/11.0;
    yv*=((bombSpeedExtra+PITCH_SPEED_AERODYN_EFFECT*1.2)+10.0)/11.0;*/

    // apply gravity
    yv+=BOMB_GRAVITY;

    // nose drift & stalling.  Too slow: dip; Too far: nose rises
    bombAngle+=(bombBreed[type].stallOverallSpeed-mag(xv,yv))*
                                  bombBreed[type].noseDriftForce;

    while(mag(xv,yv)>TERMINAL_VELOCITY)
      {
      xv*=0.98;
      yv*=0.98;
      }

    speedAsPerc=(mag(xv,yv)/TERMINAL_VELOCITY);
            // simpler system than what's needed for planes

    // affected most by turbulance when horizontal.  Faster=less stable nose
    bombAngle+=weather.turbNow*sin(flippedFacingAngle)*speedAsPerc;
    // affected most by wind when vertical.  Faster=less stable nose
    bombAngle+=weather.windNow*cos(flippedFacingAngle)*speedAsPerc;
     // going slower means wind has more effect on your position
    xv=xv+weather.windNow*cos(flippedFacingAngle)*(1.0-speedAsPerc);

    if(type!=BOMB_BUNKERBUSTER && type!=BOMB_HARPOON) // stabalizer systems
      {
      yv=yv+weather.turbNow*BOMB_LESSMASS_WINDTURB_AFFECT_SCALAR;
      xv=xv+weather.windNow*BOMB_LESSMASS_WINDTURB_AFFECT_SCALAR;
      }

    while(bombAngle<-128.0)
      bombAngle+=256.0;
    while(bombAngle>128.0)
      bombAngle-=256.0;

    if(type==BOMB_BUNKERBUSTER) // stabalizer systems
      xv/=2.0;

    x+=xv;
    y+=yv;

    if(facing==FACING_LEFT) // if moving left
      {
      if(x<PLANE_BOUNDARY_LEFT) // and offscreen left
        detonateOrVanishFlag=VANISH;
      }
    else // if moving right
      {
      if(x>PLANE_BOUNDARY_RIGHT) // and offscreen right
        detonateOrVanishFlag=VANISH;
      }

    int buildingHit;

    if(y>level.i_platformDriftHeightCurrent) // and offscreen bottom
      {
      if(type == BOMB_BUNKERBUSTER)
        {
        detonateOrVanishFlag=DETONATE;
        sound.queueSound(SFXcrash,255,x);
        buildingHit=int(PIXEL2BUILDING_X(x));
        if(buildingHit>=0 && buildingHit<MAP_W)
          level.buildingHealth[buildingHit][PLYR_TARGETS]-=bombBreed[type].damage;
        LAUNCH_EXPLOSION_PARTICLES_EX(15, x, y, 1.0, RANDOM_DEBRIS);
        }
      detonateOrVanishFlag=DETONATE;
      sound.queueSound(SFXcrash,255,x);
      }

    for(int i=0;i<MAP_W;i++)
      {
      if(level.building[i][PLYR_TARGETS]!=BLDG_NONE)
        {
        if(y>PIXELBUILDING_Y)
          {
          if(x>BUILDING2CENTERPIXEL_X(i)
               -(level.BUILDING[player.defendCorp()][level.building[i][PLYR_TARGETS]][0]->w>>1) &&
              x<BUILDING2CENTERPIXEL_X(i)
                +(level.BUILDING[player.defendCorp()][level.building[i][PLYR_TARGETS]][0]->w>>1) )
            {
            if(type == BOMB_BUNKERBUSTER)
              {
              if(detonateOrVanishFlag == KEEPALIVE)
                {
                detonateOrVanishFlag=FIRST_EXPLOSION;
                // cut the speed down to simulate "friction" of falling through
                xv=0; yv*=0.25;
                }
              }    
            else
              {
              detonateOrVanishFlag=DETONATE;
              sound.queueSound(SFXcrash,255,x);
              LAUNCH_EXPLOSION_PARTICLES_EX(15, x, y, 1.0, RANDOM_DEBRIS);
              level.buildingHealth[i][PLYR_TARGETS]-=bombBreed[type].damage;
              }
            break; // only hit 1 building
            }
          }
        }
      }
    }

  void bomb_typ::drawBomb()
    {
    int xi=((int)x)-(bombBreed[type].BOMB->w>>1)-viewport.x;
    int yi=((int)y)-(bombBreed[type].BOMB->h>>1)-viewport.y;

    if(detonateOrVanishFlag != INSIDE_BUILDING)
      {
      if(facing==FACING_RIGHT)
        rotate_sprite_v_flip(DB,bombBreed[type].BOMB,xi,yi,ftofix(bombAngle+128.0));
      else
        rotate_sprite(DB,bombBreed[type].BOMB,xi,yi,ftofix(255.0-bombAngle));
      }

    if(type==BOMB_LASER && target_timer<=0)
      {
      circlefill(DB,int(target_x)-viewport.x,int(target_y)-viewport.y,4,makecol(255,0,0));
      }
    }

  bomb_typ::bomb_typ(bomb_typ &copyBomb) // copy constructor
    { // this is used when removing a jack, because they're kept in a linear array
    x=copyBomb.x;
    y=copyBomb.y;
    xv=copyBomb.xv;
    yv=copyBomb.yv;
    target_x=copyBomb.target_x;
    target_y=copyBomb.target_y;
    target_timer=copyBomb.target_timer;
    type=copyBomb.type;
    bombAngle=copyBomb.bombAngle;
    bombSpeed=copyBomb.bombSpeed;
    detonateOrVanishFlag=copyBomb.detonateOrVanishFlag;
    }


