#include "saturn.h"

void loadBulletSamGraphicsAndData()
  {
  PALETTE the_palette;
  char bulletsamNumName[35];

  set_config_file("data//bulletsams.dat");

  for(int i=0;i<BULLETSAM_TYPES;i++)
    {
    switch(i)
      {
      MATCH_INI_NAME_TO_DEF_BULLETSAM(BULLET_BASIC)
      MATCH_INI_NAME_TO_DEF_BULLETSAM(BULLET_ADV)
      MATCH_INI_NAME_TO_DEF_BULLETSAM(SAM_BASIC)
      MATCH_INI_NAME_TO_DEF_BULLETSAM(SAM_ADV)
      MATCH_INI_NAME_TO_DEF_BULLETSAM(RESOURCE_MOMPOP)
      MATCH_INI_NAME_TO_DEF_BULLETSAM(RESOURCE_PIRATES)
      MATCH_INI_NAME_TO_DEF_BULLETSAM(RESOURCE_BIGGOV)
      MATCH_INI_NAME_TO_DEF_BULLETSAM(RESOURCE_HOME)
      }

    bulletsamBreed[i].myBulletSamType=i;
    // retrive properties from the data file, while setting them if they are
    // missing or invalid inputs (MACROS defined for this in bulletsam.h)
    GETSET_BULLETSAM_PROPERTY(visualIndex,BULLETSAM_VIS_NOBITMAP);
    GETSET_BULLETSAM_PROPERTY(color,makecol(255,255,255));
    GETSET_BULLETSAM_PROPERTY_F(specificGravityMult,0.1);
    GETSET_MIN_RANDADD_SET_BULLETSAM(minLife,maxRandAddedToMinLife,120,85);
    GETSET_MIN_RANDADD_SET_BULLETSAM(minVel,maxRandAddedToMinVel,3,4);
    GETSET_MIN_RANDADD_SET_BULLETSAM(minMovementAngle,maxRandAddedToMinMovementAngle,0,32);
    GETSET_MIN_RANDADD_SET_BULLETSAM(minAlphaDelt,maxRandAddedToMinAlphaDelt,0,0);
    GETSET_MIN_RANDADD_SET_BULLETSAM(minReloadTime,maxRandAddedToReloadTime,50,50);
    GETSET_BULLETSAM_PROPERTY(trailLength,6);
    GETSET_BULLETSAM_PROPERTY(trailWidthTop,3);
    GETSET_BULLETSAM_PROPERTY(trailWidthBot,0);
    
    if(bulletsamBreed[i].visualIndex==BULLETSAM_VIS_BITMAP)
      {
      strcat(bulletsamNumName,".pcx");
      bulletsamBreed[i].BULLETSAM=load_pcx(bulletsamNumName,the_palette);
      }
    }

  set_config_file("settings.ini"); // reset to default ini
  }

void unloadBulletSamGraphics()
  {
  for(int i=0;i<BULLETSAM_TYPES;i++)
    bulletsamBreed[i].unloadGraphics();
  }

void explodeBulletSams(int bulletsamIndex)
  {
  switch(bulletsam[bulletsamIndex].type)
    {
    case BULLET_BASIC:
    case BULLET_ADV:
      sound.queueSound(SFXcrash,255, bulletsam[bulletsamIndex].x);
      if(g_liveParticles<NUM_LIVEPARTICLES-1)
        LAUNCH_FLAME_PARTICLES_BULLETSAM(1, PARTICLE_SMOKE)
        particle[g_liveParticles].launch(bulletsam[bulletsamIndex].x,
                                     bulletsam[bulletsamIndex].y,0,0,
                                     rand()%256,255,1.0,PARTICLE_EXPLOSION_SM1);
      break;
    case SAM_BASIC:
    case SAM_ADV:
      sound.queueSound(SFXcrash,255, bulletsam[bulletsamIndex].x);
      if(g_liveParticles<NUM_LIVEPARTICLES-1)
        LAUNCH_FLAME_PARTICLES_BULLETSAM(1, PARTICLE_SMOKE)
        particle[g_liveParticles].launch(bulletsam[bulletsamIndex].x,
                                     bulletsam[bulletsamIndex].y,0,0,
                                     rand()%256,255,1.0,PARTICLE_EXPLOSION_SM1);

      LAUNCH_EXPLOSION_PARTICLES_EX(1,bulletsam[bulletsamIndex].x,bulletsam[bulletsamIndex].y,1.0,RANDOM_EXPLOSION)
      LAUNCH_EXPLOSION_PARTICLES_BULLETSAM(1,RANDOM_DEBRIS_SM)
      break;
    case RESOURCE_MOMPOP:
    case RESOURCE_PIRATES:
    case RESOURCE_BIGGOV:
    case RESOURCE_HOME:
      sound.queueSound(SFXcrash,255, bulletsam[bulletsamIndex].x);
      if(g_liveParticles<NUM_LIVEPARTICLES-1)
        LAUNCH_FLAME_PARTICLES_BULLETSAM(1, PARTICLE_SMOKE)
        particle[g_liveParticles].launch(bulletsam[bulletsamIndex].x,
                                     bulletsam[bulletsamIndex].y,0,0,
                                     rand()%256,255,1.0,PARTICLE_EXPLOSION_SM1);
      LAUNCH_EXPLOSION_PARTICLES_BULLETSAM(10,RANDOM_DEBRIS_SM)
      break;
    }
  }

bulletsam_breed_typ bulletsamBreed[BULLETSAM_TYPES];

  bulletsam_breed_typ::bulletsam_breed_typ()
    {
    BULLETSAM=NULL;
    myBulletSamType=0;
    visualIndex=BULLETSAM_VIS_NOBITMAP; // no image
    specificGravityMult=1.0;
    color=0;
    }

  void bulletsam_breed_typ::unloadGraphics()
    {
    if(visualIndex!=BULLETSAM_VIS_BITMAP)
      return;

    destroy_bitmap(BULLETSAM);
    }

bulletsam_typ bulletsam[NUM_LIVEBULLETSAMS];

/* The actual class prototype is defined inside init.h */
  bulletsam_typ::bulletsam_typ()
    {

    }

  void bulletsam_typ::reset()
    {
    detonateOrVanishFlag=KEEPALIVE;
    trail.set(6, 0, 1, makecol(0, 128, 255));
    }

  void bulletsam_typ::launch(float new_x, float new_y, float new_alpha,int new_type)
    {
    g_liveBulletSams++;
    detonateOrVanishFlag=KEEPALIVE;

    type=new_type;

    SET_TO_BULLETSAM_BREED_RANGE(life,minLife,maxRandAddedToMinLife);
    SET_TO_BULLETSAM_BREED_RANGE(speed,minVel,maxRandAddedToMinVel);
    ang=bulletsamBreed[type].minMovementAngle-
                       (bulletsamBreed[type].maxRandAddedToMinMovementAngle>>1);
    ang+=rand()%(bulletsamBreed[type].maxRandAddedToMinMovementAngle);
    SET_TO_BULLETSAM_BREED_RANGE(alphaDelta,minAlphaDelt,maxRandAddedToMinAlphaDelt);

    xv=speed*cos(ang);
    yv=speed*sin(ang);
    startX = int(new_x);
    startY = int(new_y);
    x=new_x;
    y=new_y;

    trail.set(bulletsamBreed[type].trailWidthTop,
              bulletsamBreed[type].trailWidthBot,
              bulletsamBreed[type].trailLength, makecol(0, 128, 255));
    }

  void bulletsam_typ::moveBulletSam()
    {
    float speedAsPerc;
    float littleLeft=ang-2.0;
    float littleRight=ang+2.0;

    alpha+=alphaDelta;

    if(plane.type!=PLANE_TYPES_STEALTH &&
        player.ownsPlane(plane.econ*PLANE_TYPES+plane.type)) // not dead
      {
      switch(type)
        {
        case SAM_BASIC:
          ang=atan2(plane.y-y,plane.x-x);
          break;
        case SAM_ADV:
          ang=atan2(plane.y-y,plane.x-x);
          break;
        }
      }
    else
      {
      switch(type)
        {
        case SAM_BASIC:
        case SAM_ADV:
          ang+=rand()%8;
          ang-=rand()%8;
          break;
        }

      if(y>level.i_platformDriftHeightCurrent) // run into platform
        detonateOrVanishFlag=DETONATE;
      }

    xv=speed*cos(ang);
    yv=speed*sin(ang);

    // apply gravity
    yv+=BULLETSAM_GRAVITY*bulletsamBreed[type].specificGravityMult;

    yv=yv+weather.turbNow*BULLETSAM_LESSMASS_WINDTURB_AFFECT_SCALAR;
    xv=xv+weather.windNow*BULLETSAM_LESSMASS_WINDTURB_AFFECT_SCALAR;

    x+=xv;
    y+=yv;

    if(life--<0)
      detonateOrVanishFlag=DETONATE;

    if(xv<=0) // if moving left
      {
      if(x<PLANE_BOUNDARY_LEFT) // and offscreen left
        detonateOrVanishFlag=VANISH;
      }
    else // if moving right
      {
      if(x>PLANE_BOUNDARY_RIGHT) // and offscreen right
        detonateOrVanishFlag=VANISH;
      }
    if(y<0) // disappear into the atmosphere
      detonateOrVanishFlag=VANISH;
      

    int targetWidth=(planeBreed[player.battleCorp()][plane.type][plane.econ].PLANE->w>>1);
    int targetHeight=(planeBreed[player.battleCorp()][plane.type][plane.econ].PLANE->h>>1);

    if(x>plane.x-targetWidth && x<plane.x+targetWidth &&
         y>plane.y-targetHeight && y<plane.y+targetHeight)
       {
       switch(type)
         {
         case BULLET_BASIC:
         case SAM_BASIC:
           plane.health--;
           break;
         case BULLET_ADV:
         case SAM_ADV:
           plane.health-=2;
           break;
         case RESOURCE_MOMPOP:
         case RESOURCE_PIRATES:
         case RESOURCE_BIGGOV:
         case RESOURCE_HOME:
           plane.health-=3;
           break;
         }
       if(plane.health<0)
         plane.health=0;
       plane.planeSpeed*=planeBreed[player.battleCorp()][plane.type]
                                               [plane.econ].throttleLossWhenHit;
       if(plane.health==0 &&
         player.owns[player.activePlayer].plane[player.owns[player.activePlayer].planeIndex])
         {
         if(g_liveParticles<NUM_LIVEPARTICLES-1)
           particle[g_liveParticles].launch(plane.x, plane.y,0,0,
                            rand()%256,255,1.0,PARTICLE_EXPLOSION_SM1);
         }
       viewport.shakeScreen(5+(rand()%10));
       detonateOrVanishFlag=DETONATE;
       }

    trail.addSegment(
      ((int)x),
      ((int)y),
      ang-(64.0));
    }

  void bulletsam_typ::drawBulletSam()
    {
    int xi=(int)x-viewport.x;
    int yi=(int)y-viewport.y;

    int spin_x=(speed*cos(ang))>>1;
    int spin_y=(speed*sin(ang))>>1;

    trail.draw();

    float count = 0;
    switch (bulletsamBreed[type].visualIndex)
      {
      case BULLETSAM_VIS_BITMAP:
        rotate_sprite(DB, bulletsamBreed[type].BULLETSAM, xi, yi, ftofix(ang));
        break;
      case BULLETSAM_VIS_LASER:
        drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
        set_trans_blender(255, 255, 255, 96);
        rectfill(DB, startX-(60/(life+2))-viewport.x, startY-viewport.y, xi+(60/(life+2)), yi, bulletsamBreed[type].color);
        rectfill(DB, startX-(life>>1)-viewport.x, startY-viewport.y, xi+(life>>1), yi, bulletsamBreed[type].color);
        rectfill(DB, startX-(life>>2)-viewport.x, startY-viewport.y, xi+(life>>2), yi, bulletsamBreed[type].color);
        drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
        break;
      case BULLETSAM_VIS_RAILGUN:
        vline(DB, xi, int(startY)-viewport.y, yi, bulletsamBreed[type].color);
        for (int p = int(startY); p >= y; p-=3, count++)
          vline(DB, xi-life*sin(life*count), p-viewport.y, p-viewport.y+10, makecol(0, 128+rand()%128, 255));
        break;
      default:
        line(DB,xi-spin_x,yi-spin_y,xi+spin_x,yi+spin_y,bulletsamBreed[type].color);     
      }
    }

  bulletsam_typ::bulletsam_typ(bulletsam_typ &copyBulletSam) // copy constructor
    { // this is used when removing, since they're kept in a linear array
    x=copyBulletSam.x;
    y=copyBulletSam.y;
    xv=copyBulletSam.xv;
    yv=copyBulletSam.yv;
    angDelta=copyBulletSam.angDelta;
    ang=copyBulletSam.ang;
    alphaDelta=copyBulletSam.alphaDelta;
    alpha=copyBulletSam.alpha;
    scaleDelta=copyBulletSam.scaleDelta;
    scale=copyBulletSam.scale;
    speed=copyBulletSam.speed;
    life=copyBulletSam.life;
    type=copyBulletSam.type;
    detonateOrVanishFlag=copyBulletSam.detonateOrVanishFlag;
    trail=copyBulletSam.trail;
    }


