#include "saturn.h"

/* Note: define these as externs in init.h if you want to use them elsewhere */
BITMAP *DB;
BITMAP *CURSOR;
DATAFILE *fontData;
int g_msDelayPerFrame;
int g_transparency;
int g_colorWhite;
int g_colorPitchCircle;
int colorDepth;
int g_developerMode;
int g_inGame;

int g_liveBombs;
int g_liveParticles;
int g_liveBulletSams;

int snowShade[256]; // not globally accessed, so don't put in init.h
BITMAP *FADE;
  
inline double fastsqrt(double r)
  {
  double x,y;
  double tempf;
  unsigned long *tfptr = ((unsigned long *)&tempf)+1;

  tempf = r;
  *tfptr=(0xbfcd4600-*tfptr)>>1;
  x=tempf;
  y=r*0.5;
  x*=1.5-x*x*y;  // I only need to estimate ;-)
  return x*r;
  }

float mag(float x,float y) // not gloplaney prototyped; for plane speed only
  {
  return fastsqrt(x*x+y*y);
  }

int OpenAllegro()
  {
  int fullscreen;
  int screenW, screenH;

  allegro_init();
  install_keyboard();
  install_mouse(); /* only initialize what we need*/
  install_timer();
  srand(time(NULL)); // seed random # generator, based on time

  set_config_file("settings.ini");
  /* This just indicates to Allegro that this is the file we'll be
     referring to in the upcoming get_config_[int/string/'whatever']
     commands.  Settings.ini is just a straightforward textfile.
     Open it with notepad to see how these upcoming commands work. */

  // need these pixels for the game to work.  Try to force refresh them:
  set_config_int("screen","width",1024);
  set_config_int("screen","height", 768);
 
  /* "heading", "name under that heading", default value if not found */
	screenW=get_config_int("screen","width",1024);
  screenH=get_config_int("screen","height", 768);
  fullscreen=get_config_int("screen","fullscreen", 1);
  g_msDelayPerFrame=get_config_int("game","framerest", 10);
  g_developerMode=get_config_int("game","developerMode", 0);
  level.levelsUnlocked=get_config_int("game","UnlockA", level.levelsUnlocked);
  level.shipsUnlocked=get_config_int("game","UnlockD", level.levelsUnlocked);
  if(CHEATER_CHECKSUM1(level.levelsUnlocked)!=get_config_int("game","UnlockB", -1) ||
     CHEATER_CHECKSUM2(level.levelsUnlocked)!=get_config_int("game","UnlockC", -1))
    level.levelsUnlocked=1;
  if(CHEATER_CHECKSUM1(level.shipsUnlocked)!=get_config_int("game","UnlockE", -1) ||
     CHEATER_CHECKSUM2(level.shipsUnlocked)!=get_config_int("game","UnlockF", -1))
    level.shipsUnlocked=1;
  
  if(level.levelsUnlocked>10)
    level.levelsUnlocked=10;
  else if(level.levelsUnlocked<1)
    level.levelsUnlocked=1;
  sound.maxVolumeSFX=get_config_int("audio","maxvolumesounds", 255);
  sound.maxVolumeMusic=get_config_int("audio","maxvolumemusic", 255);

  if(sound.maxVolumeSFX<1)
    sound.maxVolumeSFX=1;
  if(sound.maxVolumeMusic<1)
    sound.maxVolumeMusic=1;

  /* In case the settings file didn't exist yet, create it...
    "heading", "name under that heading", value to save with that name */
  set_config_int("screen","width",screenW);
  set_config_int("screen","height", screenH);
  set_config_int("screen","fullscreen", fullscreen);
  set_config_int("game","developerMode", g_developerMode);
  set_config_int("game","framerest", g_msDelayPerFrame);
  set_config_int("game","UnlockA", level.levelsUnlocked);
  set_config_int("game","UnlockB", CHEATER_CHECKSUM1(level.levelsUnlocked));
  set_config_int("game","UnlockC", CHEATER_CHECKSUM2(level.levelsUnlocked));
  set_config_int("game","UnlockD", level.shipsUnlocked);
  set_config_int("game","UnlockE", CHEATER_CHECKSUM1(level.shipsUnlocked));
  set_config_int("game","UnlockF", CHEATER_CHECKSUM2(level.shipsUnlocked));
  set_config_int("audio","maxvolumesounds", sound.maxVolumeSFX);
  set_config_int("audio","maxvolumemusic", sound.maxVolumeMusic);

  colorDepth=16;

  /* Try several times to set the video mode.  Each time the attempt
     fails, it will try the next nestled set of options.  A full list
     of the gfx options are available in allegro.txt within the docs
     folder. */

  set_color_depth(colorDepth);
  if(set_gfx_mode((fullscreen ? GFX_DIRECTX_ACCEL : GFX_DIRECTX_WIN), screenW, screenH, 0, 0) != 0)
    {
    screenW=800;screenH=600;
    if(set_gfx_mode((fullscreen ? GFX_DIRECTX_ACCEL : GFX_DIRECTX_WIN), screenW, screenH, 0, 0) != 0)
      {
      screenW=640;screenH=480;
      if(set_gfx_mode(GFX_AUTODETECT, screenW, screenH, 0, 0) != 0)
        {
        if(set_gfx_mode(GFX_DIRECTX_WIN, screenW,screenH, 0, 0) != 0) // didn't fit in any screen doable res, so window
          {
          allegro_message("Error setting graphics mode\n%s\n", allegro_error);
          return 1;
          }
        }
      }
    }

  g_transparency=makecol(255,0,255);
  /* Easier to check against later.  Note that in 8-bit 256 color video
     modes, index 0 is the invisible masking color for bitmaps.  In any
     other video color mode (16, 24, or 32 bit) the masking color is
     fuschia, or full red and full blue with no green.  Makecol is slightly
     inefficient, since it checks to determine which color depth we're in
     before returning an appropriate integer, so we cache it here in case
     we choose to manually process a bitmap pixel by pixel. */
  g_colorWhite=makecol(255,255,255);
   /* We use this for text color in this version of the program, so we're
      going to cache the color now.  White has a way of being handy... */
  g_colorPitchCircle=makecol(127,127,127);

  if(install_joystick(JOY_TYPE_AUTODETECT) != 0)
    {
    // don't throw a fit just because there's no joystick plugged in
    } // just using buttons, so we'll skip the calibration B.S. and analog/digital concerns

  fontData=load_datafile("data//gamefont.dat");
  if(!fontData)
    {
    allegro_message("Error finding font datafile\n%s\n", allegro_error);
    return(2);
    }

  if(!fullscreen)
    {
    set_window_title("Saturn Storm");
    set_display_switch_mode(SWITCH_BACKGROUND);
    /* Sets the window name, and tells Windows to let the program continue
       to run if the user clicks on something else.*/
    }

  sound.initializeSoundCard();
  text_mode(-1); // transparent Allegro text
   /* Giving text_mode any value besides -1 will interpret that as a
      background color to put behind text.  -1 tells Allegro to print
      the text masked, i.e. without a solid background. */

  for(int i=0;i<256;i++) // precompute, for the fading snowy effect
    snowShade[i]=makecol(i,i,i);

  return(0);
  }

int LoadMemory()
  {
  PALETTE palette;
  /* Load images to memory in a function different than that which
     sets the screen mode.  This avoids incorrectly freeing memory
     you haven't allocated, if you have to quickly close out due to
     a hardware failure of some kind. */

  level.getLevelCount();

  CURSOR = load_pcx("data//cursor.pcx",palette);
  set_mouse_sprite(CURSOR);
  set_mouse_sprite_focus(CURSOR->w>>1,CURSOR->h>>1);

   /* CURSOR is a globally scoped BITMAP *.  Here we define it and tell
     Allegro that we want to use it as the mouse cursor.  If you comment
     out the line for "set_mouse_sprite" Allegro will default to a
     regular white arrow. */

  DB = create_bitmap(SCREEN_W,SCREEN_H);
  rectfill(DB,0,0,DB->w,DB->h,0);
  /* We make the DoubleBuffer the same size as the real screen, so that we
     can draw everything to it, then blit it all to the real screen at once.
     This is done to avoid redraw flickering.
     */

  FADE=create_bitmap(SCREEN_W,SCREEN_H); // only used for snowfade effect

  jpgalleg_init();

  level.loadGraphics();
  loadBombGraphics();
  loadParticleGraphicsAndData();
  loadBulletSamGraphicsAndData();
  weather.loadGraphics();
  loadPlaneGraphicsAndData();
  sound.loadSfxAndMusic();

  viewport.reset();

  return(0);
  }

int EmptyMemory()
  {
  set_config_file("settings.ini");

  /* In case the settings file didn't exist yet, create it...
    "heading", "name under that heading", value to save with that name */
  set_config_int("screen","width",SCREEN_W);
  set_config_int("screen","height",SCREEN_H);
  set_config_int("game","framerest", g_msDelayPerFrame);
  set_config_int("game","developerMode", g_developerMode);
  set_config_int("game","UnlockA", level.levelsUnlocked);
  set_config_int("game","UnlockB", CHEATER_CHECKSUM1(level.levelsUnlocked));
  set_config_int("game","UnlockC", CHEATER_CHECKSUM2(level.levelsUnlocked));
  set_config_int("game","UnlockD", level.shipsUnlocked);
  set_config_int("game","UnlockE", CHEATER_CHECKSUM1(level.shipsUnlocked));
  set_config_int("game","UnlockF", CHEATER_CHECKSUM2(level.shipsUnlocked));
  set_config_int("audio","maxvolumesounds", sound.maxVolumeSFX);
  set_config_int("audio","maxvolumemusic", sound.maxVolumeMusic);

  destroy_bitmap(CURSOR);
  destroy_bitmap(DB); // free all image memory!
  destroy_bitmap(FADE);
  level.unloadGraphics();
  unloadBombGraphics();
  unloadParticleGraphics();
  unloadBulletSamGraphics();
  weather.unloadGraphics();
  unloadPlaneGraphics();
  sound.unloadSfxAndMusic();
  unload_datafile(fontData);

  return(0);
  }

int CloseAllegro()
  {
  remove_timer();
  remove_keyboard();
  remove_mouse();
  remove_joystick();
   /* These remove lines are actually called from allegro_exit() if needed,
      but it's good practice to put them here since it keeps you aware
      of what you've initialized. */
  allegro_exit();
  return(0);
  }

void DeliNoise(BITMAP *BMP)
  {
  int wx,wy;

  for(wx=0;wx<SCREEN_W;wx++)
    for(wy=0;wy<SCREEN_H;wy++)
      {
      if(rand()%7<=1)
        FAST_PIXEL_SNOW(BMP,wx,wy,(rand()&255));
      }
  blit(BMP,screen,0,0,0,0,SCREEN_W,SCREEN_H);
  }

void DeliUnNoise()
  {
  int wx,wy;
  int color;

  for(wx=0;wx<SCREEN_W;wx++)
    for(wy=0;wy<SCREEN_H;wy++)
      {
      if(rand()%7<=1)
        {
        color=FAST_GETPIXEL(FADE,wx,wy);
        FAST_PIXEL_TRUE(DB,wx,wy,color);
        }
      }
  DB2SCREEN
  }

void fadeBMP(BITMAP *BMP)
  {
  int w,h,ws,hs,opp;
  int col1,col2;

  if(gamemenu.abortHack)
    return;

  blit(DB,FADE,0,0,0,0,SCREEN_W,SCREEN_H);

  for(int i=0;i<5;i++)
    DeliNoise(BMP);
  blit(BMP,DB,0,0,0,0,SCREEN_W,SCREEN_H);

  for(int i=0;i<3;i++)
    DeliUnNoise();

  blit(FADE,screen,0,0,0,0,SCREEN_W,SCREEN_H);
  }
