#include "saturn.h"

level_typ level;

  level_typ::level_typ()
    {
    i_platformDriftHeightCurrent=10000;
    playersMode=P1;
    currentLevel=1;
    subLevel=1;
    goodEvilRecord=0;

    strcpy(levelTitle,"");
    displayTitleTime=0;

    gravityScalar=0.0;

    for(int i=0;i<MAP_W;i++)
      for(int p=0;p<PLAYER_NUM;p++)
        building[i][p]=BLDG_RESOURCE;

    BACKGROUND=NULL; /* initialize to NULL, because we'll check before loading
                        if it needs to be destroyed */
    levelsUnlocked=1;
    shipsUnlocked=0;

    buildingHealth_MAX[BLDG_NONE]=-1;
    buildingHealth_MAX[BLDG_RESOURCE]=10;
    buildingHealth_MAX[BLDG_REPAIR]=15;
    buildingHealth_MAX[BLDG_DEFLECTION]=20;
    buildingHealth_MAX[BLDG_STDAA]=10;
    buildingHealth_MAX[BLDG_ADVAA]=30;
    buildingHealth_MAX[BLDG_STDSAM]=10;
    buildingHealth_MAX[BLDG_ADVSAM]=30;
    buildingHealth_MAX[BLDG_CRITICALTARGET]=100;
    }

  void level_typ::getLevelCount()
    {
    char charSection[MAP_W+1];
    char charSectionLineName[20],numScrap[4];
    char charSectionFileName[22];

    levelCount=0;
    do
      {
      levelCount++;
      if(levelCount<10)
        strcpy(charSectionLineName,"data//l00");
      else if(levelCount<100)
        strcpy(charSectionLineName,"data//l0");
      else
        strcpy(charSectionLineName,"data//l");

      itoa(levelCount,numScrap,10);
      strcat(charSectionLineName,numScrap);
      strcpy(charSectionFileName,charSectionLineName);
      strcat(charSectionFileName,".dat");

      set_config_file(charSectionFileName); // Always ensure we're on the right cfg file
      }while(get_config_int("MapVals","corp", -666)!=-666);
    // keep reading levels until we hit one with no level time (returns default)
    levelCount--;
    }

  // magnitude (distance); fast-diamond approx.
  // this is used for determining if a ball is close enough to a target to
  // finish the level
  float level_typ::fastDiamondDistance(float x,float y)
    {
    if(x<0)
      {
      if(y<0)
        return(-(x+y));
      else
        return(y-x);
      }
    else
      {
      if(y<0)
        return(x-y);
      else
        return(y+x);
      }
    }

  void level_typ::loadGraphics()
    {
    PALETTE palette;
    BITMAP *scrap;
    int height,width;
    int doneHoriz,doneVert;

    /* here we load 1 long string of arrows facing right, and then
       cut it apart into different sections facing right, finally
       rotating it each of the other 3 cardinal directions, completing
       our arrow graphics set. */
    scrap=load_pcx("data//building.pcx",palette);
    for(int corp=0;corp<NUM_OF_CORPORATIONS;corp++)
      {
      for(int type=0;type<BLDG_TYPES;type++)
        {
        for(int frame=0;frame<BUILDING_DAMAGE_FRAMES;frame++)
          {
          width=BUILDING_W-1;
          height=BUILDING_H-1;
          doneHoriz=doneVert=0;
          do
            {
            if(!doneVert)
              {
              for(int wx=0;wx<width;wx++)
                {
                if(getpixel(scrap,type*BUILDING_W+wx,corp*BUILDING_H+height)
                  !=g_transparency)
                  {
                  doneVert=1;
                  break;
                  }
                }
              if(!doneVert) height--;
              }
          
            if(!doneHoriz)
              {
              for(int wy=0;wy<height;wy++)
                {
                if(getpixel(scrap,type*BUILDING_W+width,corp*BUILDING_H+wy)
                  !=g_transparency)
                  {
                  doneHoriz=1;
                  break;
                  }
                }
              if(!doneHoriz) width--;
              }
            } while((doneHoriz==0 || doneVert==0) && height>1 && width>1);

          // NOTE: MUST create a bitmap before drawing to it in memory!
          // (if loaded directly from a file, the bitmap is automatically created)
          BUILDING[corp][type][frame]=create_bitmap(width,height);
          blit(scrap,BUILDING[corp][type][frame],
                   type*BUILDING_W,corp*BUILDING_H, // from the correct corner here
                   0,0, // to top left corner of target bitmap
                   width,height); // full width and height
          }
        }
      }

    // don't leave this garbage
    destroy_bitmap(scrap);

    // reminder: LOAD_JPG isn't standard Allegro, it's from the JPGAlleg
    // library that we have linked in.  More details on this in rowdy.h.

    TOPBAR=load_pcx("data//topbar.pcx",palette);
    PITCHCROSS=load_pcx("data//pitchcross.pcx",palette);
    THROTTLEBAR=load_pcx("data//throttlebar.pcx",palette);
    STATUSBAR=load_pcx("data//statusbar.pcx",palette);

    PLATFORM=load_pcx("data//platform1.pcx",palette);

    BACKGROUND=create_bitmap(1,1);
    /* just make SOMETHING, so when
       we destroy it in level load to make room for the next one, we won't
       be destroying unallocated memory. */
    }

  void level_typ::unloadGraphics()
    {
    for(int corp=0;corp<NUM_OF_CORPORATIONS;corp++)
      for(int type=0;type<BLDG_TYPES;type++)
        for(int frame=0;frame<BUILDING_DAMAGE_FRAMES;frame++)
          destroy_bitmap(BUILDING[corp][type][frame]);

    destroy_bitmap(BACKGROUND);
    destroy_bitmap(PLATFORM);
    destroy_bitmap(TOPBAR);
    destroy_bitmap(PITCHCROSS);
    destroy_bitmap(THROTTLEBAR);
    destroy_bitmap(STATUSBAR);
    }

  void level_typ::resetToOpeningLevel(int startLevel)
    {
    player.resetRounds();
    if(level.playersMode==P1)
      {
      player.owns[P1].reset();
      player.owns[P2].reset(); // do something different with P2 AI money?
      }
    else
      {
      player.owns[P1].reset();
      player.owns[P2].reset();
      }

    if(level.playersMode==P1)
      {
      currentLevel=startLevel;
      subLevel=1;
      }
    else
      {
      switch(player.mp_map)
        {
        case 1: currentLevel=1; subLevel=1; break;
        case 2: currentLevel=1; subLevel=2; break;
        case 3: currentLevel=1; subLevel=3; break;
        case 4: currentLevel=2; subLevel=1; break;
        case 5: currentLevel=2; subLevel=2; break;
        case 6: currentLevel=2; subLevel=3; break;
        case 7: currentLevel=3; subLevel=1; break;
        case 8: currentLevel=3; subLevel=2; break;
        case 9: currentLevel=3; subLevel=3; break;
        case 10: currentLevel=4; subLevel=1; break;
        }
      }
    loadBldg();
    }

  int level_typ::loadBldg()
    {
    char charSection[MAP_W+1];
    char charSectionLineName[20],numScrap[4];
    char charSectionFileName[22];
    PALETTE palette; // needed to load_jpg or load_pcx

    /* Here we'll use the INI parser, the very same one used to load
       game settings at startup, to load level datastructures.  Notice
       the breaking of a string into individual characters, and then
       filtering those characters into array representations for the level. */

    RESET_GLOBAL_COUNTS
    weather.randomizeRain();

    switch(currentLevel)
      {
      case 1:
        switch(subLevel)
          {
          case 1: INCREASE_UPTO(levelsUnlocked,1) break;
          case 2: INCREASE_UPTO(levelsUnlocked,2) break;
          case 3: INCREASE_UPTO(levelsUnlocked,3) break;
          }
        break;
      case 2:
        switch(subLevel)
          {
          case 1: INCREASE_UPTO(levelsUnlocked,4) break;
          case 2: INCREASE_UPTO(levelsUnlocked,5) break;
          case 3: INCREASE_UPTO(levelsUnlocked,6) break;
          }
        break;
      case 3:
        switch(subLevel)
          {
          case 1: INCREASE_UPTO(levelsUnlocked,7) break;
          case 2: INCREASE_UPTO(levelsUnlocked,8) break;
          case 3: INCREASE_UPTO(levelsUnlocked,9) break;
          }
        break;
      case 4:
        INCREASE_UPTO(levelsUnlocked,10)
        break;
      }

    strcpy(charSectionLineName,"data//l");

    itoa(currentLevel,numScrap,10);
    strcat(charSectionLineName,numScrap);
    strcpy(charSectionFileName,charSectionLineName);
    
    itoa(subLevel,numScrap,10);
    strcat(charSectionLineName,numScrap);
    strcpy(charSectionFileName,charSectionLineName);

    strcat(charSectionFileName,".dat");

    set_config_file(charSectionFileName); // Always ensure we're on the right cfg file

    if(get_config_int("MapVals","corp", -666)==-666)
      {
      // requested level file doesn't exist
      return 1;
      }

    strcpy(charSectionFileName,charSectionLineName);
    strcat(charSectionFileName,".jpg");

    strcpy(charSectionFileName,
           get_config_string("MapVals","backgroundjpg", charSectionFileName));
    
    if(load_jpg(charSectionFileName,palette)==NULL)
      {
      // requested level image doesn't exist
      return 2;
      }

    destroy_bitmap(BACKGROUND);
    BACKGROUND=NULL;
    BACKGROUND=load_jpg(charSectionFileName,palette);
    if(BACKGROUND==NULL)
      {
      strcpy(charSectionFileName,"data//sky1.jpg"); // if we can't find it
      BACKGROUND=load_jpg(charSectionFileName,palette);
      }
    //set_config_string("MapVals","backgroundjpg", charSectionFileName);
    //save_jpg(charSectionFileName,BACKGROUND,palette);

    strcpy(levelTitle,
           get_config_string("MapVals","levelTitle", "Untitled Level"));
    displayTitleTime=LEVEL_TITLE_DISPLAY_CYCLES;

    weather.reset();

    weather.windSpeedBase=get_config_float("MapVals","windSpeedCenter", 1.0);
    weather.windSpeedMaxDev=get_config_float("MapVals","windSpeedMax", 1.0);
    weather.windSpeedRateOfDev=get_config_float("MapVals","windSpeedDeviation", 1.0);
    weather.turbSpeedBase=get_config_float("MapVals","turbulanceCenter", 1.0);
    weather.turbSpeedMaxDev=get_config_float("MapVals","turbulanceMax", 1.0);
    weather.turbSpeedRateOfDev=get_config_float("MapVals","turbulanceDeviation", 1.0);

    gravityScalar=get_config_float("MapVals","gravityPower", 1.0);
    if(playersMode==P1)
      player.setDefendCorp(get_config_int("MapVals","corp", CORP_PIRATES));

    platformDriftRapidity=0.8;
    platformDriftDistance=7.5;
    platformFalling=0;
    platformDriftHeightCenter=GRID_START_CORNER_Y;
    platformDriftAngle=0.0;

    LIMIT_WITHIN(weather.windSpeedBase,UNREASONABLE_THREASHOLD)
    LIMIT_WITHIN(weather.windSpeedMaxDev,UNREASONABLE_THREASHOLD)
    LIMIT_WITHIN(weather.windSpeedRateOfDev,UNREASONABLE_THREASHOLD)
    LIMIT_WITHIN(weather.turbSpeedBase,UNREASONABLE_THREASHOLD)
    LIMIT_WITHIN(weather.turbSpeedMaxDev,UNREASONABLE_THREASHOLD)
    LIMIT_WITHIN(weather.turbSpeedRateOfDev,UNREASONABLE_THREASHOLD)
    LIMIT_WITHIN(gravityScalar,UNREASONABLE_THREASHOLD)

    // replace any missing or incorrect variables in the level file:
    set_config_string("MapVals","levelTitle", levelTitle);
    set_config_float("MapVals","windSpeedCenter", weather.windSpeedBase);
    set_config_float("MapVals","windSpeedMax", weather.windSpeedMaxDev);
    set_config_float("MapVals","windSpeedDeviation", weather.windSpeedRateOfDev);
    set_config_float("MapVals","turbulanceCenter", weather.turbSpeedBase);
    set_config_float("MapVals","turbulanceMax", weather.turbSpeedMaxDev);
    set_config_float("MapVals","turbulanceDeviation", weather.turbSpeedRateOfDev);

    set_config_float("MapVals","gravityPower", gravityScalar);
    if(playersMode==P1)
      set_config_int("MapVals","corp", player.corp[P2]);

    plane.facing=FACING_LEFT; // will get flipped
    
    strcpy(charSection,get_config_string("Buildings","layout",
                                  "_____ABA_S_$_$_ARRRA_RRB_$_$_S_ABA________"));

    for(int i=0;i<MAP_W;i++)
      for(int p=0;p<PLAYER_NUM;p++) // identical bases for 2 player mode
        {
        buildingReload[i]=0; // reset all gun timers
        switch(charSection[i])
          {
          case '$': building[i][p]=BLDG_RESOURCE; break;
          case 'R': building[i][p]=BLDG_REPAIR; break;
          case 'D': building[i][p]=BLDG_DEFLECTION; break;
          case 'A': building[i][p]=BLDG_STDAA; break;
          case 'B': building[i][p]=BLDG_ADVAA; break;
          case 'S': building[i][p]=BLDG_STDSAM; break;
          case 'T': building[i][p]=BLDG_ADVSAM; break;
          case 'X': building[i][p]=BLDG_CRITICALTARGET; break;
          case '_': building[i][p]=BLDG_NONE; break;          }

        buildingHealth[i][p]=buildingHealth_MAX[building[i][p]];
        if(level.playersMode==P1)
          {
          switch(player.corp[P2])
            {
            case CORP_MOMPOP:
              buildingHealth[i][p]*=1;
              break;
            case CORP_PIRATES:
              buildingHealth[i][p]*=2;
              break;
            case CORP_BIGGOV:
              buildingHealth[i][p]*=3;
              break;
            case CORP_HOME:
              buildingHealth[i][p]*=4;
              break;
            }
          }
        else
          buildingHealth[i][p]*=2;
        }

    destroy_bitmap(PLATFORM);
    switch(player.defendCorp())
      {
      case CORP_MOMPOP:
        PLATFORM=load_pcx("data//platform1.pcx",palette);
        break;
      case CORP_PIRATES:
        PLATFORM=load_pcx("data//platform2.pcx",palette);
        break;
      case CORP_BIGGOV:
        PLATFORM=load_pcx("data//platform3.pcx",palette);
        break;
      case CORP_HOME:
        PLATFORM=load_pcx("data//platform4.pcx",palette);
        break;
      case CORP_BUG:
        PLATFORM=load_pcx("data//platform5.pcx",palette);
        break;
      }

    // open bombing menu
    sound.loadLevelSong(player.defendCorp());
    gamemenu.runLoadoutMenu(1);
    return 0; // no errors
    }

  void level_typ::moveBuildings()
    {
    if(platformFalling)
      {
      if(platformDriftHeightCenter<4000.0) // just don't let it fall FOREVER
        {
        platformDriftHeightCenter+=platformFreefall_yv;
        platformFreefall_yv+=BOMB_GRAVITY;
        }
      }

    for(int i=0;i<MAP_W;i++)
      {
      if(building[i][PLYR_TARGETS]==BLDG_NONE)
        continue;
      if(buildingHealth[i][PLYR_TARGETS]<=0)
        {
        if(building[i][PLYR_TARGETS]==BLDG_CRITICALTARGET && !platformFalling)
          {
          for(int ii=0;ii<SFX_NUM;ii++) // silence all other sounds
            sound.toPlay[SFXPARAM_VOLUME][i]=0;
          if(mag((BUILDING2CENTERPIXEL_X(i))-plane.x,platformDriftHeightCurrent-plane.y)<NUKE_EXPLOSION_RANGE)
            plane.health=0; // BOOM.  Dead.  Sucker
          sound.queueSound(SFXplatBoom,255, BUILDING2CENTERPIXEL_X(i));
          sound.queueSound(SFXplatFall,255, BUILDING2CENTERPIXEL_X(i));
          LAUNCH_EXPLOSION_PARTICLES_EX(1, float(BUILDING2CENTERPIXEL_X(i)),
             float(i_platformDriftHeightCurrent), 0.5, PARTICLE_NUKE_BLAST)
          for(int ii=0;ii<MAP_W;ii++)
            {
            LAUNCH_EXPLOSION_PARTICLES_EX(1, float(BUILDING2CENTERPIXEL_X(ii)-(rand()%30)+(rand()%30)),
             platformDriftHeightCurrent-(rand()%30), 1.0, RANDOM_EXPLOSION)
            LAUNCH_EXPLOSION_PARTICLES_EX(8, float(BUILDING2CENTERPIXEL_X(ii)-(rand()%30)+(rand()%30)),
             platformDriftHeightCurrent-(rand()%30), 1.0, PARTICLE_SPARK)
            }
          platformFalling=1;
          plane.planeSpeed/=3.0; // cut engine power from disruption
          viewport.shakeScreen(100+(rand()%30));
          }
        building[i][PLYR_TARGETS]=BLDG_NONE;
        }

      if(platformFalling)
        return;

      switch(building[i][PLYR_TARGETS])
        {
        case BLDG_STDAA:
          if(buildingReload[i]<=0)
            {
            sound.queueSound(SFXrattleOut,255, BUILDING2CENTERPIXEL_X(i));
      
            buildingReload[i]+=bulletsamBreed[BULLET_BASIC].minReloadTime+
             (rand()%(bulletsamBreed[BULLET_BASIC].maxRandAddedToReloadTime));
            if(g_liveBulletSams<NUM_LIVEBULLETSAMS-1)
                bulletsam[g_liveBulletSams].launch(float(BUILDING2CENTERPIXEL_X(i)),
                float(i_platformDriftHeightCurrent-(BUILDING[player.defendCorp()]
                                     [building[i][PLYR_TARGETS]][0]->h)),
                255.0,BULLET_BASIC);
            }
          else buildingReload[i]--;
          break;
        case BLDG_ADVAA:
          if(buildingReload[i]<=0)
            {
            sound.queueSound(SFXrattleOut,255, bulletsam[i].x);
      
            buildingReload[i]+=bulletsamBreed[BULLET_ADV].minReloadTime+
             (rand()%(bulletsamBreed[BULLET_ADV].maxRandAddedToReloadTime));
            for(int ii=0; ii<2; ii++)
              if(g_liveBulletSams<NUM_LIVEBULLETSAMS-1)
                bulletsam[g_liveBulletSams].launch(float(BUILDING2CENTERPIXEL_X(i)),
                float(i_platformDriftHeightCurrent-(BUILDING[player.defendCorp()]
                                     [building[i][PLYR_TARGETS]][0]->h)),
                255.0,BULLET_ADV);
            }
          else buildingReload[i]--;
          break;
        case BLDG_STDSAM:
          if(buildingReload[i]<=0)
            {
            sound.queueSound(SFXrocketOut,255, BUILDING2CENTERPIXEL_X(i));
      
            buildingReload[i]+=bulletsamBreed[SAM_BASIC].minReloadTime+
             (rand()%(bulletsamBreed[SAM_BASIC].maxRandAddedToReloadTime));
            if(g_liveBulletSams<NUM_LIVEBULLETSAMS-1)
               bulletsam[g_liveBulletSams].launch(float(BUILDING2CENTERPIXEL_X(i)),
               float(i_platformDriftHeightCurrent-(BUILDING[player.defendCorp()]
                        [building[i][PLYR_TARGETS]][0]->h)),
                255.0,SAM_BASIC);
            }
          else buildingReload[i]--;
          break;
        case BLDG_ADVSAM:
          if(buildingReload[i]<=0)
            {
            sound.queueSound(SFXrocketOut,255, BUILDING2CENTERPIXEL_X(i));

            buildingReload[i]+=bulletsamBreed[SAM_ADV].minReloadTime+
             (rand()%(bulletsamBreed[SAM_ADV].maxRandAddedToReloadTime));
            for(int ii=0; ii<2; ii++)
              if(g_liveBulletSams<NUM_LIVEBULLETSAMS-1)
              bulletsam[g_liveBulletSams].launch(float(BUILDING2CENTERPIXEL_X(i)),
                float(i_platformDriftHeightCurrent-(BUILDING[player.defendCorp()]
                       [building[i][PLYR_TARGETS]][0]->h)),
                255.0,SAM_ADV);
            }
          else buildingReload[i]--;
          break;
        case BLDG_RESOURCE:
          if(buildingReload[i]<=0)
            {
            sound.queueSound(SFXzap,255, bulletsam[i].x);
            buildingReload[i]+=
             bulletsamBreed[RESOURCE_MOMPOP+player.defendCorp()].minReloadTime+
             (rand()%(bulletsamBreed[RESOURCE_MOMPOP+
               player.defendCorp()].maxRandAddedToReloadTime));
            if(g_liveBulletSams<NUM_LIVEBULLETSAMS-1)
              bulletsam[g_liveBulletSams].launch(float(BUILDING2CENTERPIXEL_X(i)),
                float(i_platformDriftHeightCurrent-(BUILDING[player.defendCorp()]
                       [building[i][PLYR_TARGETS]][0]->h)),
                255.0,RESOURCE_MOMPOP+player.defendCorp());
            }
          else buildingReload[i]--;
          break;
        case BLDG_DEFLECTION:
          int peakPoint_x=BUILDING2CENTERPIXEL_X(i);
          int peakPoint_y=i_platformDriftHeightCurrent-
              (BUILDING[player.defendCorp()][building[i][PLYR_TARGETS]][0]->h);
          if(buildingReload[i]<=0)
            {
            for(int ii=0;ii<g_liveBombs;ii++)
              {
              if(mag(bomb[ii].x-peakPoint_x,bomb[ii].y-peakPoint_y)<DEFLECTION_TOWER_RANGE
                 && bomb[ii].type!=BOMB_PLASTIC)
                {
                LAUNCH_EXPLOSION_PARTICLES_EX(40, peakPoint_x, peakPoint_y, 1.0, PARTICLE_SPARK);
                LAUNCH_EXPLOSION_PARTICLES_EX(3, peakPoint_x, peakPoint_y, 1.0, PARTICLE_SMOKE);
                sound.playSound(SFXelectricDeath,255,SCREEN_W>>1);
                jitterCircle(peakPoint_x, peakPoint_y, 20, 20, 4, makecol(190,200,255));
                ripple(peakPoint_x, peakPoint_y, 10, 10, 40);
                bomb[ii].xv=-(1.4+(rand()%3))*bomb[ii].xv;
                bomb[ii].yv=-(1.4+(rand()%3))*bomb[ii].yv;
                bomb[ii].x+=bomb[ii].xv;
                bomb[ii].y+=(1.4+(rand()%3))*bomb[ii].yv;
                buildingReload[i]+=20+(rand()%120); // will it stop it next time?
                 // discharge, & require reload
                }
              }
            }
          else buildingReload[i]--;
          break;
        }
      }
    }

  int level_typ::levelIsOver()
    {
    int foundOne;
    int p1killedAllBldg=0;
    int p2killedAllBldg=0;
    int p1killedAllShip=0;
    int p2killedAllShip=0;

    if(platformFalling==1)
      {
      for(int i=0;i<MAP_W;i++)
        building[i][player.activePlayer]=BLDG_NONE;
      platformFalling=0;
      }

    if(level.playersMode==P2)
      {
      if(player.activePlayer!=P1)
        return NO_WIN;
      if(player.roundsLeft==-1)
        { // sum of assets, minus buildings you left standing
        if(player.owns[P1].worth(P1)==player.owns[P2].worth(P2))
          return DRAW_WIN;
        else if(player.owns[P1].worth(P1)>player.owns[P2].worth(P2))
          return P1_WIN;
        return P2_WIN;
        }
      }

    foundOne=0;
    for(int i=0;i<MAP_W;i++)
      if(building[i][P1]!=BLDG_NONE) // all P1 target buildings destroyed
        {
        foundOne=1;
        break;
        }

    if(!foundOne) // all P1 target buildings destroyed
      p1killedAllBldg=1;

    foundOne=0;
    for(int i=0;i<MAP_W;i++)
      if(building[i][P2]!=BLDG_NONE) // all P2 target buildings destroyed
        {
        foundOne=1;
        break;
        }

    if(!foundOne) // all P2 target buildings destroyed
      p2killedAllBldg=1;

    foundOne=0;
    for(int i=0;i<PLANE_KINDS;i++)
      if(player.owns[P2].plane[i]) // all P1 planes destroyed
        {
        foundOne=1;
        break;
        }

    if(!foundOne)
      p1killedAllShip=1;

    foundOne=0;
    for(int i=0;i<PLANE_KINDS;i++)
      if(player.owns[P1].plane[i]) // all P1 planes destroyed
        {
        foundOne=1;
        break;
        }

    if(!foundOne)
      p2killedAllShip=1;

    if(p1killedAllShip+p2killedAllShip+
           p1killedAllBldg+p2killedAllBldg==0)
      return NO_WIN;
    else if(p1killedAllBldg>p2killedAllBldg)
      return P1_WIN;
    else if(p2killedAllBldg>p1killedAllBldg)
      return P2_WIN;
    else if(p1killedAllShip>p2killedAllShip)
      return P1_WIN;
    else if(p2killedAllShip>p1killedAllShip)
      return P2_WIN;
    else
      {
      if(player.owns[P1].worth(P1)==player.owns[P2].worth(P2))
        return DRAW_WIN;
      else if(player.owns[P1].worth(P1)>player.owns[P2].worth(P2))
        return P1_WIN;
      return P2_WIN;
      }
    }

  void level_typ::drawBackgroundAndBuildings()
    {
    int xt=0;

    /* We could use draw_sprite, but we would need to
       make sure there was no (255,0,255) transparency in the bitmap.
       (draw_sprite assumes trasparency masking) */
    blit(BACKGROUND,DB,0,0,0,0,BACKGROUND->w,BACKGROUND->h);

    platformDriftAngle+=platformDriftRapidity;
    if(platformDriftAngle>=256.0)
      platformDriftAngle-=256.0;
    platformDriftHeightCurrent=platformDriftHeightCenter+
                                  platformDriftDistance*cos(platformDriftAngle);

    i_platformDriftHeightCurrent=(int)platformDriftHeightCurrent;

    draw_sprite(DB,PLATFORM,-viewport.x,i_platformDriftHeightCurrent-23-viewport.y);
    draw_sprite(DB,PLATFORM,SCREEN_W-viewport.x,i_platformDriftHeightCurrent-23-viewport.y);
    draw_sprite(DB,PLATFORM,(SCREEN_W<<1)-viewport.x,i_platformDriftHeightCurrent-23-viewport.y);

    int buildingLeft=PIXEL2BUILDING_X(viewport.x-BUILDING_W);
    int buildingRight=PIXEL2BUILDING_X(viewport.x+SCREEN_W+BUILDING_W);
    if(buildingLeft<0)
      buildingLeft=0;
    if(buildingRight>MAP_W)
      buildingRight=MAP_W;

    for(int i=buildingLeft;i<buildingRight;i++)
      {
      if(building[i][PLYR_TARGETS]!=BLDG_NONE)
        {
        draw_sprite(DB,BUILDING[player.defendCorp()][building[i][PLYR_TARGETS]][0],
           BUILDING2CENTERPIXEL_X(i)-(BUILDING[player.defendCorp()][building[i]
                                    [PLYR_TARGETS]][0]->w>>1)-viewport.x,
           PIXELBUILDING_Y-viewport.y);
        }
      }
    }

  void level_typ::drawSBar()
    {
    int startSpotX;

    draw_sprite(DB,TOPBAR,(((DB->w)-(TOPBAR->w))>>1),0);
    draw_sprite(DB,STATUSBAR,(((DB->w)-(STATUSBAR->w))>>1),(TOPBAR->h));

    startSpotX=SBAR_INFO_RESERVES_X-
             ((plane.dumbBombs*(bombBreed[BOMB_DUMB].BOMB->h+BOMB_SPACING))>>1);

    for(int i=0;i<plane.dumbBombs;i++)
      {
      rotate_sprite(DB,bombBreed[0].BOMB,
          startSpotX+i*(bombBreed[0].BOMB->h+BOMB_SPACING),SBAR_INFO_RESERVES_Y,
          itofix(64));
      }

    startSpotX=SBAR_INFO_SPECIAL_X-
       ((plane.specialBombs*(bombBreed[player.specialBombArmed].BOMB->h+BOMB_SPACING))>>1);

    for(int i=0;i<plane.specialBombs;i++)
      {
      rotate_sprite(DB,bombBreed[player.specialBombArmed].BOMB,
          startSpotX+i*(bombBreed[player.specialBombArmed].BOMB->h+BOMB_SPACING),
          SBAR_INFO_SPECIAL_Y,itofix(64));
      }

    // pitch thingie
    int leftOfPitchCircle,rightOfPitchCircle;
    rectfill(DB,SBAR_PITCH_X,SBAR_PITCH_Y,SBAR_PITCH_X2,SBAR_PITCH_Y2,0);

    if(plane.planeAngle>-64.0 && plane.planeAngle<64.0)
      {
      leftOfPitchCircle=(int)(plane.planeAngle/4.0);
      rightOfPitchCircle=leftOfPitchCircle;

      if(plane.facing==FACING_RIGHT)
        {
        rightOfPitchCircle+=(int)(weather.turbNow*plane.speedAsPerc*PITCH_TURB_EXAGG);
        leftOfPitchCircle-=(int)(weather.turbNow*plane.speedAsPerc*PITCH_TURB_EXAGG);
        }
      else
        {
        rightOfPitchCircle-=(int)(weather.turbNow*plane.speedAsPerc*PITCH_TURB_EXAGG);
        leftOfPitchCircle+=(int)(weather.turbNow*plane.speedAsPerc*PITCH_TURB_EXAGG);
        }

      LIMIT_WITHIN(leftOfPitchCircle,15);
      LIMIT_WITHIN(rightOfPitchCircle,15);

      line(DB,SBAR_PITCH_X,SBAR_PITCH_YM-leftOfPitchCircle,SBAR_PITCH_X2,
              SBAR_PITCH_YM-rightOfPitchCircle,g_colorPitchCircle);
      floodfill(DB,SBAR_PITCH_XM,SBAR_PITCH_Y2-1,g_colorPitchCircle);
      }
    else
      {
      float fakeAngle;

      if(plane.planeAngle<-64.0)
        fakeAngle=-128-plane.planeAngle;
      else // if(player.pitchAngle>64.0)
        fakeAngle=128-plane.planeAngle;

      leftOfPitchCircle=(int)(fakeAngle/4.0);
      rightOfPitchCircle=leftOfPitchCircle;

      if(plane.facing==FACING_RIGHT)
        {
        rightOfPitchCircle+=(int)(weather.turbNow*plane.speedAsPerc*PITCH_TURB_EXAGG);
        leftOfPitchCircle-=(int)(weather.turbNow*plane.speedAsPerc*PITCH_TURB_EXAGG);
        }
      else
        {
        rightOfPitchCircle-=(int)(weather.turbNow*plane.speedAsPerc*PITCH_TURB_EXAGG);
        leftOfPitchCircle+=(int)(weather.turbNow*plane.speedAsPerc*PITCH_TURB_EXAGG);
        }

      LIMIT_WITHIN(leftOfPitchCircle,15);
      LIMIT_WITHIN(rightOfPitchCircle,15);

      line(DB,SBAR_PITCH_X,SBAR_PITCH_YM+leftOfPitchCircle,SBAR_PITCH_X2,
                  SBAR_PITCH_YM+rightOfPitchCircle,g_colorPitchCircle);
      floodfill(DB,SBAR_PITCH_XM,SBAR_PITCH_Y+1,g_colorPitchCircle);
      }
    draw_sprite(DB,PITCHCROSS,SBAR_PITCH_X,SBAR_PITCH_Y);

    int throttleBarSize;

    throttleBarSize=(int)(((float)THROTTLEBAR->w)*plane.speedAsPerc);
    blit(THROTTLEBAR,DB,throttleBarSize,0,SBAR_THROTTLE_X+throttleBarSize,
              SBAR_THROTTLE_Y,THROTTLEBAR->w-throttleBarSize,THROTTLEBAR->h);

    int windDist=(int)(weather.windNow*TURB_WIND_EXAGG_WINSOCK);

    // windsock
    line(DB,SBAR_WINDSOCK_X,SBAR_WINDSOCK_Y-1,SBAR_WINDSOCK_X+
     (int)(windDist*weather.windSpeedBase),SBAR_WINDSOCK_Y-1,g_colorPitchCircle);
    line(DB,SBAR_WINDSOCK_X,SBAR_WINDSOCK_Y,SBAR_WINDSOCK_X+
     (int)(windDist*weather.windSpeedMaxDev),SBAR_WINDSOCK_Y,g_colorWhite);
    line(DB,SBAR_WINDSOCK_X,SBAR_WINDSOCK_Y+1,SBAR_WINDSOCK_X+
     (int)(windDist*weather.windSpeedRateOfDev),SBAR_WINDSOCK_Y+1,g_colorPitchCircle);

    int turbOffset=(int)(weather.turbNow*PITCH_TURB_EXAGG);

    line(DB,SBAR_TURB_X-((int)(weather.turbSpeedBase*TURB_WIND_EXAGG_WINSOCK)),SBAR_TURB_Y+turbOffset,
        SBAR_TURB_X+((int)(weather.turbSpeedBase*TURB_WIND_EXAGG_WINSOCK)),SBAR_TURB_Y-turbOffset,g_colorPitchCircle);
    line(DB,SBAR_TURB_X-((int)(weather.turbSpeedRateOfDev*TURB_WIND_EXAGG_WINSOCK)),SBAR_TURB_Y-turbOffset,
        SBAR_TURB_X+((int)(weather.turbSpeedRateOfDev*TURB_WIND_EXAGG_WINSOCK)),SBAR_TURB_Y+turbOffset,g_colorWhite);
    line(DB,SBAR_TURB_X-((int)(weather.turbSpeedMaxDev*TURB_WIND_EXAGG_WINSOCK)),SBAR_TURB_Y+turbOffset,
        SBAR_TURB_X+((int)(weather.turbSpeedMaxDev*TURB_WIND_EXAGG_WINSOCK)),SBAR_TURB_Y-turbOffset,g_colorPitchCircle);
    
    // energy display
    int leftSide=((SBAR_PITCH_X+SBAR_PITCH_X2)>>1)-
           (((planeBreed[player.battleCorp()][plane.type][plane.econ].healthMax-1)>>1)*5)+3;
    for(int i=0;i<planeBreed[player.battleCorp()][plane.type][plane.econ].healthMax;i++)
      rect(DB,leftSide+5*(i-1),SBAR_PITCH_Y2+3,leftSide+5*i-2,SBAR_PITCH_Y2+7,g_colorWhite);
    for(int i=0;i<plane.health;i++)
      rectfill(DB,leftSide+5*(i-1),SBAR_PITCH_Y2+3,leftSide+5*i-2,SBAR_PITCH_Y2+7,g_colorWhite);

    textprintf_right(DB, GAMEFONT, SBAR_MONEY_X1,SBAR_MONEY_Y, g_colorWhite, "$%i gM",player.owns[P1].cash);
    textprintf(DB, GAMEFONT, SBAR_MONEY_X2,SBAR_MONEY_Y, g_colorWhite, "$%i gM",player.owns[P2].cash);

    int spot_x=282,spot_y=67;
    int color=(player.activePlayer==P1 ? g_colorWhite : 0);
    switch(player.corp[P1])
      {
      case CORP_MOMPOP:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, ">P1 PMR>");
        break;
      case CORP_PIRATES:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, ">P1 ScP>");
        break;
      case CORP_BIGGOV:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, ">P1 J.C>");
        break;
      case CORP_HOME:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, ">P1 ZHT>");
        break;
      case CORP_BUG:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, ">P1 Ins>");
        break;
      }
    spot_x=741;
    color=(player.activePlayer==P2 ? g_colorWhite : 0);
    switch(player.corp[P2])
      {
      case CORP_MOMPOP:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, "<%s PMR<",
          level.playersMode==P2 ? "P2" : "AI");
        break;
      case CORP_PIRATES:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, "<%s ScP<",
          level.playersMode==P2 ? "P2" : "AI");
        break;
      case CORP_BIGGOV:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, "<%s J.C<",
          level.playersMode==P2 ? "P2" : "AI");
        break;
      case CORP_HOME:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, "<%s ZHT<",
          level.playersMode==P2 ? "P2" : "AI");
        break;
      case CORP_BUG:
        textprintf_centre(DB, GAMETINYFONT, spot_x,spot_y, color, "<%s Ins<",
          level.playersMode==P2 ? "P2" : "AI");
        break;
      }

    if(level.playersMode==P2)
      {
      if(player.mp_rounds)
        {
        textprintf(DB, GAMEFONT, 4,SCREEN_H-41, 0,
         "Round %i of %i",player.mp_rounds-player.roundsLeft,player.mp_rounds);
        textprintf(DB, GAMEFONT, 2,SCREEN_H-43, makecol(200,0,0),
         "Round %i of %i",player.mp_rounds-player.roundsLeft,player.mp_rounds);
        }
      else
        {
        textprintf(DB, GAMEFONT, 4,SCREEN_H-41, 0,
         "Annihilation Mode");
        textprintf(DB, GAMEFONT, 2,SCREEN_H-43, makecol(200,0,0),
         "Annihilation Mode");
        }
      }

    if(displayTitleTime>0)
      {
      int glow=16-(displayTitleTime&15);
      if(glow<0)glow=-glow;

      textout_centre(DB, GAMEFONT, levelTitle,
                (SCREEN_W>>1),BACKGROUND->h-167, 0);
      textout_centre(DB, GAMEFONT, levelTitle,
                (SCREEN_W>>1),BACKGROUND->h-170, makecol(((glow<<4)+85)&255,0,0));
      displayTitleTime--;
      }
    }

