#include "saturn.h"

void loadParticleGraphicsAndData()
{
  PALETTE the_palette;
  
  // The name of the particle-type we're defining.
  char particleNumName[35];
  char bitmapFilename[256];

  set_config_file("data//particles.dat");

  // AWESOME Hack! PARTICLE_TYPES, the last element in the types enum,
  // easily represents the number of definitions in the enum.
  // Iterates over all the particle types, loading in each one's
  // configuration.
  for(int i=0;i<PARTICLE_TYPES;i++)
  {
    // Nice Hack! All these fine-ass macros expand to various case statements.
    // Sets the section name (particleNumName) to the enumeration name of the
    // currente particle number
    switch(i)
    {
      MATCH_INI_NAME_TO_DEF(PARTICLE_SOLID)
      MATCH_INI_NAME_TO_DEF(PARTICLE_BITMAP)
      MATCH_INI_NAME_TO_DEF(PARTICLE_SMOKE)
      MATCH_INI_NAME_TO_DEF(PARTICLE_DEBRIS_CLOUD)
      MATCH_INI_NAME_TO_DEF(PARTICLE_NAPALM)
      MATCH_INI_NAME_TO_DEF(PARTICLE_FLAME)
      MATCH_INI_NAME_TO_DEF(PARTICLE_EXPLOSION1)
      MATCH_INI_NAME_TO_DEF(PARTICLE_EXPLOSION2)
      MATCH_INI_NAME_TO_DEF(PARTICLE_EXPLOSION3)
      MATCH_INI_NAME_TO_DEF(PARTICLE_EXPLOSION4)
      MATCH_INI_NAME_TO_DEF(PARTICLE_EXPLOSION_SM1)
      MATCH_INI_NAME_TO_DEF(PARTICLE_SPARK)
      MATCH_INI_NAME_TO_DEF(PARTICLE_EMP_RING)
      MATCH_INI_NAME_TO_DEF(PARTICLE_EMP_BOLT)
      MATCH_INI_NAME_TO_DEF(PARTICLE_SHARD)
      MATCH_INI_NAME_TO_DEF(PARTICLE_DEBRIS1)
      MATCH_INI_NAME_TO_DEF(PARTICLE_DEBRIS2)
      MATCH_INI_NAME_TO_DEF(PARTICLE_DEBRIS3)
      MATCH_INI_NAME_TO_DEF(PARTICLE_DEBRIS4)
      MATCH_INI_NAME_TO_DEF(PARTICLE_DEBRIS_SM1)
      MATCH_INI_NAME_TO_DEF(PARTICLE_DEBRIS_SM2)
      MATCH_INI_NAME_TO_DEF(PARTICLE_BOMBLET)
      MATCH_INI_NAME_TO_DEF(PARTICLE_NUKE_BLAST)
      MATCH_INI_NAME_TO_DEF(PARTICLE_NUKE_BOMBLET)
    }

    particleBreed[i].myParticleType=i;
    
    // retrive properties from the data file, while setting them if they are
    // missing or invalid inputs (MACROS defined for this in particle.h)
    GETSET_PARTICLE_PROPERTY(visualIndex,PARTICLE_VIS_NOBITMAP);  
    GETSET_PARTICLE_PROPERTY(animNumFrames, 0);
    GETSET_PARTICLE_PROPERTY(animLoop, 0);
    GETSET_MIN_RANDADD_SET(animDelay,maxRandAddedToAnimDelay,0,0);
    GETSET_PARTICLE_PROPERTY(color,makecol(255,255,255));
    GETSET_PARTICLE_PROPERTY_F(specificGravityMult,1.0);
    GETSET_PARTICLE_PROPERTY_F(bounceDampening,-0.8);
    GETSET_PARTICLE_PROPERTY_F(weatherAffect,2.5);
    GETSET_PARTICLE_PROPERTY(canCollide, 1);
    GETSET_MIN_RANDADD_SET(minLife,maxRandAddedToMinLife,120,85);
    GETSET_MIN_RANDADD_SET_F(minVel,maxRandAddedToMinVel,3,9);
    GETSET_MIN_RANDADD_SET(minMovementAngle,maxRandAddedToMinMovementAngle,0,128);
    GETSET_MIN_RANDADD_SET(minRotVel,maxRandAddedToMinRotVel,-64,128);
    GETSET_PARTICLE_PROPERTY(angleLock,0);
    GETSET_MIN_RANDADD_SET(minAlphaDelt,maxRandAddedToMinAlphaDelt,0,0);
    GETSET_MIN_RANDADD_SET_F(minScaleDelt,maxRandAddedToMinScaleDelt,0,0);
    GETSET_PARTICLE_PROPERTY(trail,0);
    GETSET_PARTICLE_PROPERTY(trailJitter,5);

    if (particleBreed[i].animNumFrames <= 1)
      particleBreed[i].animNumFrames = 1;

    if (particleBreed[i].visualIndex >= PARTICLE_VIS_BITMAP )
    {
      strcpy(bitmapFilename, "data/");
      strcat(bitmapFilename, particleNumName);
      strcat(bitmapFilename, (particleBreed[i].visualIndex == PARTICLE_VIS_ALPHA) ? ".tga" : ".pcx");
      
      if (particleBreed[i].visualIndex == PARTICLE_VIS_ALPHA )
        set_color_conversion(COLORCONV_NONE);
        
      if (!(particleBreed[i].PARTICLE = load_bitmap(bitmapFilename,the_palette)))
        { allegro_message("Could not load file %s. Aborting.", bitmapFilename); exit(-1); }
      
      set_color_conversion(COLORCONV_TOTAL);
    }
  }

  set_config_file("settings.ini"); // reset to default ini
}

void unloadParticleGraphics()
  {
  for(int i=0;i<PARTICLE_TYPES;i++)
    particleBreed[i].unloadGraphics();
  }

// Special function for throwing debris when a bunker buster
// first hits a building.
void explodeBunkerBusterParticles(int bombIndex)
{
  LAUNCH_EXPLOSION_PARTICLES_EX(5, bomb[bombIndex].x, bomb[bombIndex].y-bomb[bombIndex].yv-5, 0.5, PARTICLE_SMOKE)
  LAUNCH_EXPLOSION_PARTICLES_EX(15, bomb[bombIndex].x, bomb[bombIndex].y-bomb[bombIndex].yv-5, 1.0, RANDOM_DEBRIS_SM)
}  

void explodeParticles(int bombIndex)
{
  int stopPoint_x; // for distance from EMP drop point

  switch(bomb[bombIndex].type)
  {
    case BOMB_HE:
      LAUNCH_EXPLOSION_PARTICLES(3, RANDOM_EXPLOSION)
      LAUNCH_EXPLOSION_PARTICLES(10, PARTICLE_SPARK)
      LAUNCH_EXPLOSION_PARTICLES(6, PARTICLE_BOMBLET)
      // fall through
    case BOMB_PLASTIC:
    case BOMB_LASER: // paid for these, so give'em a little show
      LAUNCH_EXPLOSION_PARTICLES(10, PARTICLE_SPARK)
      LAUNCH_EXPLOSION_PARTICLES(3, PARTICLE_BOMBLET)
      // fall through
    case BOMB_HARPOON:
    case BOMB_DUMB:
      sound.queueSound(SFXbomb,255,bomb[bombIndex].x);
      LAUNCH_EXPLOSION_PARTICLES(1, RANDOM_EXPLOSION)
      LAUNCH_EXPLOSION_PARTICLES(20, PARTICLE_SPARK)
      LAUNCH_FLAME_PARTICLES(4, PARTICLE_SMOKE)
      LAUNCH_EXPLOSION_PARTICLES(2, PARTICLE_BOMBLET)
      break;
    case BOMB_BUNKERBUSTER:
      sound.queueSound(SFXbomb,255,bomb[bombIndex].x);
      for (int i = -40; i <= 40; i+= 20)
      {
        LAUNCH_EXPLOSION_PARTICLES_EX(1, bomb[bombIndex].x+i, bomb[bombIndex].y, 0.25, RANDOM_EXPLOSION)
        LAUNCH_EXPLOSION_PARTICLES_EX(1, bomb[bombIndex].x+i, bomb[bombIndex].y, 1.0, PARTICLE_DEBRIS_CLOUD)
      }  
      break;
    case BOMB_CLUSTER:
      sound.queueSound(SFXcrash,255,bomb[bombIndex].x);
      LAUNCH_EXPLOSION_PARTICLES(35, PARTICLE_SPARK)
      LAUNCH_EXPLOSION_PARTICLES(1, RANDOM_EXPLOSION)
      LAUNCH_EXPLOSION_PARTICLES(20, PARTICLE_BOMBLET)
      break;
    case BOMB_EMP:
      sound.queueSound(SFXmajorZap,255,bomb[bombIndex].x);
      sound.queueSound(SFXcrash,255,bomb[bombIndex].x);
      LAUNCH_EXPLOSION_PARTICLES(1, PARTICLE_EMP_RING)
      LAUNCH_EXPLOSION_PARTICLES(20, PARTICLE_EMP_BOLT)
      stopPoint_x=PIXEL2BUILDING_X(int(bomb[bombIndex].x));
      for(int i=0;i<stopPoint_x;i++) // add ridiculous delays to weapon reloads
        {
        level.buildingReload[i]+=(MAP_W-(stopPoint_x-i))*EMP_JAM_DELAY_BY_MAP_W;
        }
      for(int i=stopPoint_x;i<MAP_W;i++) // add ridiculous delays to weapon reloads
        {
        level.buildingReload[i]+=(MAP_W-(i-stopPoint_x))*EMP_JAM_DELAY_BY_MAP_W;
        }
      g_liveBulletSams=0; // DESTROYS ALL AA BULLETS AND SAMS!
       // that alone makes it worth carrying in later levels!
      break;
    case BOMB_NAPALM:
      sound.queueSound(SFXnapalm,255,bomb[bombIndex].x);
      LAUNCH_EXPLOSION_PARTICLES(10, PARTICLE_NAPALM)
      LAUNCH_FLAME_PARTICLES(4, PARTICLE_SMOKE)
      break;
    case BOMB_NUKE: // a.k.a. the "HOLY F*CK" effect
      if(mag(bomb[bombIndex].x-plane.x,bomb[bombIndex].y-plane.y)<NUKE_EXPLOSION_RANGE)
        plane.health=0; // BOOM.  Dead.  Sucker

      stopPoint_x=PIXEL2BUILDING_X(int(bomb[bombIndex].x));
      for(int i=0;i<stopPoint_x;i++) // add ridiculous delays to weapon reloads
        {
        level.buildingHealth[i][PLYR_TARGETS]-=(MAP_W-(stopPoint_x-i))*NUKE_HURT_BY_MAP_W;
        }
      for(int i=stopPoint_x;i<MAP_W;i++) // add ridiculous delays to weapon reloads
        {
        level.buildingHealth[i][PLYR_TARGETS]-=(MAP_W-(i-stopPoint_x))*NUKE_HURT_BY_MAP_W;
        }
      sound.queueSound(SFXplatBoom,255,bomb[bombIndex].x);
      LAUNCH_EXPLOSION_PARTICLES_EX(100, bomb[bombIndex].x+RAND_POS_NEG(600), bomb[bombIndex].y-(rand()%450), 1.0, PARTICLE_NUKE_BOMBLET)
      LAUNCH_EXPLOSION_PARTICLES_EX(1, bomb[bombIndex].x, bomb[bombIndex].y, 1.0, PARTICLE_NUKE_BLAST)
      break;
   }
}

particle_breed_typ particleBreed[PARTICLE_TYPES];

  particle_breed_typ::particle_breed_typ()
    {
    PARTICLE=NULL;
    myParticleType=0;
    visualIndex=PARTICLE_VIS_NOBITMAP; // no image
    bounceDampening=-0.8;
    specificGravityMult=1.0;
    weatherAffect = 2.5;
    color=0;
    }

  void particle_breed_typ::unloadGraphics()
    {
    if(visualIndex!=PARTICLE_VIS_BITMAP)
      return;

    destroy_bitmap(PARTICLE);
    }

particle_typ particle[NUM_LIVEPARTICLES];

/* The actual class prototype is defined inside init.h */
  particle_typ::particle_typ()
    {

    }

  void particle_typ::reset()
    {
    detonateOrVanishFlag=KEEPALIVE;
    }

  void particle_typ::launch(float new_x, float new_y, float init_xv,float init_yv,
                                     float new_ang,float new_alpha,float new_scale,int new_type)
  {
    float new_speed, new_vectorAng;

    g_liveParticles++;
    detonateOrVanishFlag = KEEPALIVE;
    type = new_type;
    x = new_x + init_xv * 5.0;
    y = new_y + init_yv * 5.0;
    xv = init_xv;
    yv = init_yv;
    ang = new_ang;
    alpha = new_alpha;
    scale = new_scale;

    SET_TO_PARTICLE_BREED_RANGE(life,minLife,maxRandAddedToMinLife);
    SET_TO_PARTICLE_BREED_RANGE_F(new_speed,minVel,maxRandAddedToMinVel);
    SET_TO_PARTICLE_BREED_RANGE(angDelta,minRotVel,maxRandAddedToMinRotVel);
    SET_TO_PARTICLE_BREED_RANGE(new_vectorAng,minMovementAngle,maxRandAddedToMinMovementAngle);
    SET_TO_PARTICLE_BREED_RANGE(alphaDelta,minAlphaDelt,maxRandAddedToMinAlphaDelt);
    SET_TO_PARTICLE_BREED_RANGE_F(scaleDelta,minScaleDelt,maxRandAddedToMinScaleDelt);
    SET_TO_PARTICLE_BREED_RANGE(animDelay,animDelay,maxRandAddedToAnimDelay);
    
    ang = new_vectorAng;
    xv=xv+new_speed*cos(new_vectorAng);
    yv=yv+new_speed*sin(new_vectorAng);
    
    animCounter = 0;
    animFrame = 0;
    animNumLoops = 0;

    if (particleBreed[type].trail)
      partTrail.init(x, y, particleBreed[type].trailJitter, 5, particleBreed[type].trail, -16);
  }

  void particle_typ::moveParticle()
  {
    if (particleBreed[type].animNumFrames > 1)
    {
      animCounter++;
      if (animCounter >= animDelay)
      {
        animFrame++;
        animCounter = 0;
        if (animFrame >= particleBreed[type].animNumFrames)
          { animFrame = 0; animNumLoops++; }
        
        if (animNumLoops >= particleBreed[type].animLoop)
        {
          detonateOrVanishFlag = (type != PARTICLE_NUKE_BLAST) ? VANISH : DETONATE;
          if (type == PARTICLE_NUKE_BLAST)
            viewport.shakeScreen(50+(rand()%50));
        }  
      }
    }  

    if (particleBreed[type].trail)
      partTrail.update(x, y);

    alpha+=alphaDelta;
    
    // kill the particle if it's completely faded out
    if (alpha <= 0.0)
      detonateOrVanishFlag = VANISH;
    
    scale+=scaleDelta;
    
    // kill the particle if it's getting too small
    if (scale < 0.01) // tolerance
     detonateOrVanishFlag=VANISH;

    // The INSIDE_BUILDING flag is used by napalm particles.
    // If this is set, we "stick" to the current surface, by not moving the particle.
    if (detonateOrVanishFlag != INSIDE_BUILDING)
    {
      x+=xv;
      y+=yv;
  
      // apply gravity
      yv+=PARTICLE_GRAVITY*particleBreed[type].specificGravityMult;
  
      yv=yv+weather.turbNow*particleBreed[type].weatherAffect;
      xv=xv+weather.windNow*particleBreed[type].weatherAffect;
  
      ang+=angDelta;
      if (particleBreed[type].angleLock)
        ang = atan2(yv, xv);
    }  
    else if(type == PARTICLE_NAPALM)
      {
      int buildingHit=int(PIXEL2BUILDING_X(x));
      if(buildingHit>=0 && buildingHit<MAP_W)
        {
        if(rand()%50==1)
          level.buildingHealth[buildingHit][PLYR_TARGETS]-=bombBreed[BOMB_NAPALM].damage;
        if(level.buildingHealth[buildingHit][PLYR_TARGETS]<=0)
          {
          level.buildingHealth[buildingHit][PLYR_TARGETS]=0;
          detonateOrVanishFlag=DETONATE; // burned away it's target.
          return;
          }
        }
      }
    
    if(life--<0)
      detonateOrVanishFlag = (type == PARTICLE_BOMBLET || type == PARTICLE_NUKE_BLAST) ? DETONATE : VANISH;

    if(xv<=0) // if moving left
      {
      if(x<PLANE_BOUNDARY_LEFT) // and offscreen left
        detonateOrVanishFlag=VANISH;
      }
    else // if moving right
      {
      if(x>PLANE_BOUNDARY_RIGHT) // and offscreen right
        detonateOrVanishFlag=VANISH;
      }

    if(type==PARTICLE_NAPALM)
      {
      if(rand()%100==1)
        {
        LAUNCH_EXPLOSION_PARTICLES_EX(1,x,y,1.0,PARTICLE_SMOKE)
        }
      }

    if(y>=level.i_platformDriftHeightCurrent)
      {
      y=level.i_platformDriftHeightCurrent;
      if (type == PARTICLE_BOMBLET || type == PARTICLE_NUKE_BOMBLET)
        detonateOrVanishFlag = DETONATE;
      else if (type == PARTICLE_NAPALM)
        {
        xv=0.0;
        y=level.i_platformDriftHeightCurrent;
        detonateOrVanishFlag = INSIDE_BUILDING; // napalm is sticking to a surface
        }
      else
      // bouncing off platform
        yv=yv*particleBreed[type].bounceDampening; // bounce
      }

    if (particleBreed[type].canCollide)
    {
    for(int i=0;i<MAP_W;i++)
      {
        if(level.building[i][PLYR_TARGETS]!=BLDG_NONE)
        {
          if(y>PIXELBUILDING_Y)
          {
            if(x>BUILDING2CENTERPIXEL_X(i)
                 -(level.BUILDING[player.defendCorp()][level.building[i][PLYR_TARGETS]][0]->w>>1) &&
                x<BUILDING2CENTERPIXEL_X(i)
                  +(level.BUILDING[player.defendCorp()][level.building[i][PLYR_TARGETS]][0]->w>>1) )
            {
              if (type == PARTICLE_BOMBLET || type == PARTICLE_NUKE_BOMBLET)
                {
                detonateOrVanishFlag = DETONATE;
                sound.queueSound(SFXcrash,255,x);
                LAUNCH_EXPLOSION_PARTICLES_EX(3, x, y, 1.0, RANDOM_DEBRIS);
                LAUNCH_EXPLOSION_PARTICLES_EX(6, x, y, 1.0, PARTICLE_SPARK);
                level.buildingHealth[i][PLYR_TARGETS]-=bombBreed[BOMB_CLUSTER].damage;
                }
              else if (type == PARTICLE_NAPALM)
                {
                detonateOrVanishFlag = INSIDE_BUILDING; // napalm is sticking to a building
                }
              else
              {
                xv=xv*particleBreed[type].bounceDampening; // bounce diag
                yv=yv*particleBreed[type].bounceDampening; // bounce
                life>>=1; // cut life in half, so it won't last long if trapped
              }
              break; // only hit 1 building
            }
          }
        }  
      }
    }  
}

  void particle_typ::drawParticle()
  {
    int xi=(int)x-viewport.x;
    int yi=(int)y-viewport.y;
    
    if(xi<-300 || xi>SCREEN_W+300)
      return; // don't draw what's offscreen too far

    int spin_x=4*cos(ang);
    int spin_y=4*sin(ang);

    if (particleBreed[type].trail)
      partTrail.draw(xi, yi, particleBreed[type].color);

    if (particleBreed[type].visualIndex >= PARTICLE_VIS_BITMAP)
    {
      int frameW = particleBreed[type].PARTICLE->w/particleBreed[type].animNumFrames;
      BITMAP *currFrame = create_sub_bitmap(particleBreed[type].PARTICLE,
        frameW*animFrame, 0,
        frameW, particleBreed[type].PARTICLE->h);
        
      int spriteW = (int)((float)(currFrame->w)*scale);
      int spriteH = (int)((float)(currFrame->h)*scale);
         
      xi-=(spriteW>>1);
      yi-=(spriteH>>1);
  
      // Modified to handle alpha-blended particles
      switch(particleBreed[type].visualIndex)
      {
        case PARTICLE_VIS_BITMAP:
          // Don't stretch or rotate at the same time.
          if (scale != 1.0)
            stretch_sprite(DB, currFrame, xi, yi,
            spriteW, spriteH);
          else
            rotate_sprite(DB, currFrame, xi, yi, ftofix(ang));
          break;
        case PARTICLE_VIS_ALPHA:
          if (scale == 1.0)
            alpha_trans_sprite(DB, currFrame, xi, yi, (int)alpha);
          else
            stretch_alpha_trans_sprite(DB, currFrame, xi, yi,
            spriteW, spriteH, (int)alpha);
          break;
      }
      destroy_bitmap(currFrame);
    }  
    else if (particleBreed[type].visualIndex == PARTICLE_VIS_EMP)
    {
      jitterCircle(xi, yi, scale-20, 20, 4, particleBreed[type].color);
      ripple(xi, yi, (int)scale, (int)scale, 40);
    }
    else
    {
      if (alpha < 255)
      { drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0); set_trans_blender(255, 255, 255, int(alpha)); }
      line(DB,xi-spin_x,yi-spin_y,xi+spin_x,yi+spin_y,particleBreed[type].color);
    }
    
    drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
    
    // huge-ass flash if a nuke gets set off
    if (type == PARTICLE_NUKE_BLAST)
    {
      if (rand()%2 == 0)
        {
        weather.nukeRain=1;
        clear_to_color(DB,makecol(255, 255, 255));
        }
    }
}

particle_typ::particle_typ(particle_typ &copyParticle) // copy constructor
{ // this is used when removing a jack, because they're kept in a linear array
  x=copyParticle.x;
  y=copyParticle.y;
  xv=copyParticle.xv;
  yv=copyParticle.yv;
  angDelta=copyParticle.angDelta;
  ang=copyParticle.ang;
  alphaDelta=copyParticle.alphaDelta;
  alpha=copyParticle.alpha;
  life=copyParticle.life;
  type=copyParticle.type;
  detonateOrVanishFlag=copyParticle.detonateOrVanishFlag;
}



void thinTrail::init(float x, float y, int ijitter, float iweather, int idelay, int ialphadelt)
{
  numSegs = 1;
  counter = 0;
  jitter = ijitter;
  weatherAffect = iweather;
  newSegDelay = idelay;
  alphaDelt = ialphadelt;
  seg[0].x = x; seg[0].y = y; seg[0].intensity = 255;
}  

void thinTrail::update(float x, float y)
{
  counter++;
      
  if (counter % newSegDelay == 0 && numSegs < NUM_THIN_TRAIL_SEGMENTS)
  {
    seg[numSegs].x = x + ((jitter == 0) ? 0 : RAND_POS_NEG(jitter));
    seg[numSegs].y = y + ((jitter == 0) ? 0 : RAND_POS_NEG(jitter));
    seg[numSegs].intensity = 255;
    numSegs++;
  }
      
  for (int i = 0; i < numSegs; i++)
  {
    seg[i].x+=weather.turbNow*5;
    seg[i].y+=weather.windNow*5;

    if (seg[i].intensity > 0)
      seg[i].intensity += alphaDelt;
    else if (seg[i].intensity <= 0)
      seg[i].intensity = 0;
  }
}

void thinTrail::draw(int px, int py, int color)
{
  drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);       
  for (int i = 0; i <= numSegs-1; i++)
  {
    set_trans_blender(255, 255, 255, seg[i].intensity);
    
    line(DB, int(seg[i].x)-viewport.x, int(seg[i].y)-viewport.y,
      (i == numSegs-1) ? px : int(seg[i+1].x)-viewport.x,
      (i == numSegs-1) ? py : int(seg[i+1].y)-viewport.y, color);
  }  

  drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0); 
}

// "Detail" value is the distance in angle between the principal points;
// It should divide 256; the smaller the number, the more points.
void jitterCircle(int x, int y, float r, int jitter, int detail, int color)
{
   float px=r+x, py=y, px2, py2;
   for (fix i = 0; i < 256; i+=4)
   {
      px2 = r*cos(i+8)+x+RAND_POS_NEG(jitter);
      py2 = r*sin(i+8)+y+RAND_POS_NEG(jitter);

      line(DB, int(px), int(py), int(px2), int(py2), color);

      px = px2; py = py2;
   }
}

// Callback function used by do_ellipse().
void rippleCallback(BITMAP *bmp, int x, int y, int d)
{
  // Unpack the bitfield
  int cx = d & 0xFFF;
  int cy = (d & 0xFFF000) >> 12;
  int w = (d & 0xFF000000) >> 24;
  
  // Get the slope of the line
  float theta = atan2((float)(y-cy), (float)(x-cx));
  float ax = (float)(x) - cos(theta)*w;
  float ay = (float)(y) - sin(theta)*w;
  
  line(bmp, x, y, (int)ax, (int)ay, getpixel(bmp, (int)ax, (int)ay));
}  

// the ripple effect.
void ripple(int x, int y, int rx, int ry, int w)
{
  // The callback function has no knowledge of the ellipse's center, or the
  // ripple width. So we munge these into a bitfield and ship it off as the
  // d parameter. (the bitfield is 0x WW XXX YYY)
  do_ellipse(DB, x, y, (int)rx, (int)ry, x + (4096*y) + (16777216*w), rippleCallback);
}


