#include "saturn.h"

player_typ player;

  own_typ::own_typ()
    {
    reset();
    }

  void own_typ::reset()
    {
    int i;
    for(i=0;i<PLANE_KINDS;i++)
      plane[i]=0;
    for(i=0;i<BOMB_KINDS;i++)
      bomb[i]=0;

    planeIndex=bombIndex=0;

    if(level.playersMode==P1)
      cash=SINGLE_PLAYER_START_MONEY;
    else
      {
      cash=player.mp_funds;
      }

    plane[planeIndex]=1;
    bomb[bombIndex]=1;
    }

  int own_typ::worth(int myID)
    {
    int i;
    int worth=cash;

    for(i=0;i<PLANE_KINDS;i++)
      worth+=plane[i]*planeBreed[player.corp[myID]][i%PLANE_TYPES][i/PLANE_TYPES].cost;

    for(i=0;i<BOMB_KINDS;i++)
      worth+=bomb[i]*bombBreed[i].cost;

    for(i=0;i<MAP_W;i++) // how many buildings did I leave standing?
      {
      if(level.building[i][myID]!=BLDG_NONE)
        worth-=level.buildingHealth[i][myID]; // BIG sin!
      }

    return worth;
    }

  player_typ::player_typ()
    {
    keyboardHeld=NO_KEY_PRESS;
    actionWaiting=ACTION_NONE;
    specialBombArmed=BOMB_DUMB;
    bombReloadTime=0;
    activePlayer=P1;
    corp[P1]=CORP_PIRATES;
    corp[P2]=CORP_MOMPOP;
    roundsLeft=ANNIHILATION_MODE;
    mp_map=1;
    mp_rounds=5;
    mp_funds=2000;
    }

  // input system to react when a player releases a key
  // note that it's more responsive control if a game reactions
  // when a key is pushed down, instead.  This isn't much more
  // difficult to implement (remember: no holding down keys!),
  // so consider doing it as an exercise

  void player_typ::resetRounds()
    {
    if(player.mp_rounds!=0)
      roundsLeft=player.mp_rounds-1;
    else
      roundsLeft=ANNIHILATION_MODE;
    }

  int player_typ::battleCorp()
    {
    return corp[activePlayer];
    }

  int player_typ::defendCorp()
    {
    return corp[!activePlayer];
    }

  void player_typ::setDefendCorp(int newCorp)
    {
    activePlayer=P1; // single player assumed if we call setDefendCorp
    corp[P1]=CORP_HOME;
    corp[P2]=newCorp;
    }

  void player_typ::switchPlayers()
    {
    PALETTE palette;

    if(activePlayer==P1)
      activePlayer=P2;
    else
      activePlayer=P1;

    sound.loadLevelSong(player.defendCorp());

    destroy_bitmap(level.PLATFORM);
    switch(defendCorp())
      {
      case CORP_MOMPOP:
        level.PLATFORM=load_pcx("data//platform1.pcx",palette);
        break;
      case CORP_PIRATES:
        level.PLATFORM=load_pcx("data//platform2.pcx",palette);
        break;
      case CORP_BIGGOV:
        level.PLATFORM=load_pcx("data//platform3.pcx",palette);
        break;
      case CORP_HOME:
        level.PLATFORM=load_pcx("data//platform4.pcx",palette);
        break;
      case CORP_BUG:
        level.PLATFORM=load_pcx("data//platform5.pcx",palette);
        break;
      }

    // to be fair... think of it like the player changed scene facing
    weather.windSpeedBase=-weather.windSpeedBase;
    }

  int player_typ::activePlaneIndex()
    {
    return owns[activePlayer].planeIndex;
    }

  int player_typ::activeBombIndex()
    {
    return owns[activePlayer].bombIndex;
    }

  void player_typ::findActivePlaneIndex()
    {
    if(ownsPlane(owns[activePlayer].planeIndex))
      return;

    for(int i=0;i<PLANE_KINDS;i++)
      {
      if(ownsPlane(i))
        {
        owns[activePlayer].planeIndex=i;
        }
      }
    }

  void player_typ::findActiveBombIndex()
    {
    if(ownsBomb(owns[activePlayer].bombIndex))
      return;

    owns[activePlayer].bombIndex=BOMB_DUMB;
    // default to worst, since others are only 1 time use!!!
    /*for(int i=0;i<BOMB_TYPES;i++)
      {
      if(ownsBomb(i))
        owns[activePlayer].bombIndex=i;
      }*/
    }

  void player_typ::setActivePlaneIndex(int index)
    {
    owns[activePlayer].planeIndex=index;
    }

  void player_typ::setActiveBombIndex(int index)
    {
    owns[activePlayer].bombIndex=index;
    }

  int player_typ::ownsPlane(int index)
    {
    return owns[activePlayer].plane[index];
    }

  void player_typ::addPlane(int index)
    {
    owns[activePlayer].plane[index]++;
    }

  void player_typ::takePlane(int index)
    {
    owns[activePlayer].plane[index]--;
    }

  int player_typ::ownsBomb(int index)
    {
    return owns[activePlayer].bomb[index];
    }

  void player_typ::addBomb(int index)
    {
    owns[activePlayer].bomb[index]++;
    }

  void player_typ::takeBomb(int index)
    {
    owns[activePlayer].bomb[index]--;
    }

  int player_typ::afford(int cost)
    {
    return (owns[activePlayer].cash>=cost);
    }

  void player_typ::deduct(int cost)
    {
    owns[activePlayer].cash-=cost;
    }
  
  int player_typ::takeInput()
    {
    int mx,my; // mouse mickeys (movements since last frame)

    if(bombReloadTime>0)
      bombReloadTime--;

    actionWaiting=ACTION_NONE;

    poll_joystick();

    get_mouse_mickeys(&mx,&my); // take, even if not used, or they accumulate

    if(keyboardHeld>0)
      keyboardHeld--;

    if(g_developerMode)
      {
      if(key[KEY_F3]) // level skip forward
        {
        while(key[KEY_F3]) ;
        gamemenu.abortHack=ABORTHACK_1P;
        return OFF;
        }
      }

    if(key[KEY_LEFT] || JOY_L)
      {
      if(plane.facing==FACING_LEFT)
        onBit(actionWaiting,ACTION_SPEEDUP);
      else
        onBit(actionWaiting,ACTION_SPEEDDOWN);

      }
    else if(key[KEY_RIGHT] || JOY_R)
      {
      if(plane.facing==FACING_LEFT)
        onBit(actionWaiting,ACTION_SPEEDDOWN);
      else
        onBit(actionWaiting,ACTION_SPEEDUP);
      }

    if(key[KEY_UP] || JOY_U)
      {
      onBit(actionWaiting,ACTION_PITCHDOWN);
      }
    else if(key[KEY_DOWN] || JOY_D)
      {
      onBit(actionWaiting,ACTION_PITCHUP);
      }

    if(JOY_B(0) || key[KEY_SPACE])
      {
      if(!bombReloadTime)
        {
        bombReloadTime=bombBreed[BOMB_DUMB].dropBombReloadTime;
        onBit(actionWaiting,ACTION_DROPDUMB);
        }
      }
    if(JOY_B(1) || key[KEY_ENTER])
      {
      if(!bombReloadTime)
        {
        bombReloadTime=bombBreed[specialBombArmed].dropBombReloadTime;
        onBit(actionWaiting,ACTION_DROPSPECIAL);
        }
      }

    return actionWaiting;
    }
