#ifndef SATURN_H
#define SATURN_H
/* The ifndef is used to avoid these includes being defined more than once */

#include <allegro.h>  // needed for every allegro program
#include <winalleg.h> // needed for every Windows allegro program
#include <jpgalleg.h> // Need to install JPGalleg for this to be found

#include "alphasprites.h" // Needed for alpha sprite routines

/*
  JPGalleg is a community plugin that allows you to use JPG images just
  as allegro imports PCX, TGA, and BMP files.  To use it, the package
  must be installed (I found it through www.allegro.cc in the Libraries
  section: http://www.allegro.cc/resource/index.php?id=1 . Allegro.cc is
  Allegro's community website), <jpgalleg.h> must be included, the project
  must be linked to C:\mine\devcpp\Lib\libjpgal.a (or whereever that file is)
  BEFORE liballeg.a (separate with a space in Dev-C++).  Also, in the
  program initialization, jpgalleg_init(); needs to be called.

  Direct link to download JPGAlleg:
  http://www.allegro.cc/depot/project.php?_id=522

  I used this in the tutorial game (A.) because it's easy to install, and if
  you're going to use Allegro you should expose yourself to how to how
  libraries for it and (B.) it makes the otherwise HUGE background images tiny.
*/

// Include all class specific includes here, so there's no mess:

#define GAMEFONT ((FONT *)(fontData[dat_ssFont].dat)) // cast it!
#define GAMETINYFONT ((FONT *)(fontData[dat_ssTiny].dat)) // cast it!


enum {CORP_MOMPOP,CORP_PIRATES,CORP_BIGGOV,CORP_HOME,CORP_BUG,NUM_OF_CORPORATIONS};
enum {ABORTHACK_OFF,ABORTHACK_1P,ABORTHACK_1P_EXTRAMISSION,
                    ABORTHACK_2P_1WINS,ABORTHACK_2P_2WINS,ABORTHACK_2P_DRAWWINS,
                    ABORTHACK_BREAKMENUS};

/* we'll call this to copy Double Buffer to the screen */
#define DB2SCREEN                             \
   blit(DB,screen,0,0,0,0,SCREEN_W,SCREEN_H);
#define DB2SCREEN_WITH_MOUSE                  \
   {                                          \
   line(DB,mouse_x,0,mouse_x,SCREEN_H,makecol(20,120,30));\
   line(DB,0,mouse_y,SCREEN_W,mouse_y,makecol(20,120,30));\
   show_mouse(DB);                            \
   DB2SCREEN                                  \
   show_mouse(NULL);                          \
   }
/* Double buffering, or drawing to a memory bitmap before
   blitting the entire image to the video memory/screen,
   is a good idea because it's (a.) generally faster and
   (b.) avoids flickering from the player witnessing the
   redraw in progress [background blit prior to characters
   being drawn on top of it].  The show mouse line is added
   before and after the blit call because bad things can
   happen if the mouse is on a memory bitmap when that
   bitmap is drawn to.  The blit command is doing this:

   blit(DB,                   // blit FROM the global BITMAP *, DoubleBuffer
        screen,               // TO the video memory (this is always "screen")
        0,0,                  // from the top left corner of DB
        0,0,                  // to the top left corner of "screen"
        SCREEN_W,SCREEN_H);   // the width and height of the screen

   Note that when programming with allegro: "screen" is always a globally
   recognized BITMAP pointer to the video memory, and SCREEN_W and SCREEN_H
   will always be globally available as int's with the screen's width and
   height in pixels.
   */

#define LMB (mouse_b & 1)
#define RMB (mouse_b & 2)
#define MMB (mouse_b & 4)
/* mouse_b is another allegro global.  When the mouse is initialized,
   mouse_b stores the button states for the mouse.  These little LMB
   and RMB define macros make it so you don't have to remember which
   is which.  The middle button sends a value of "3," which can also
   of course be achieved by holding down both mouse buttons. */

#define COLOR_BLACK 0 // true no matter what color depth you're in
 /* unless, of course, you're in 8 bit 256 color, and remap index 0... */

#define CENTER_OF_SCREEN (SCREEN_W>>1) // for centralized sound effects, esp.

 // useful for debugging program progress (marks flags in a log file)
#define DEBUG_SCORE__(name,num) set_config_file("log.txt"); \
   set_config_int("progress",name,num);

#define RESET_GLOBAL_COUNTS g_liveParticles=g_liveBombs=0;

#include "init.h"
#include "alphatrail.h"
#include "plane.h"
#include "bomb.h"
#include "player.h"
#include "level.h"
#include "menu.h"
#include "gamemenu.h"
#include "ssfont.h"
#include "bulletsam.h"
#include "particle.h"
#include "weather.h"
#include "viewport.h"
#include "sound.h"

void drawGameFrame(int showOnScreen);

#endif

