#include "saturn.h"

sound_typ sound;

  sound_typ::sound_typ()
    {
    LEVEL_SONG=NULL;
    maxVolumeMusic=127;
    maxVolumeSFX=160;
    }

  void sound_typ::initializeSoundCard()
    {
    // we'll use this to track the playing of sounds
    // without such a system, sounds may be played multiple times
    // in the same frame
    for(int i=0;i<SFXPARAM_NUM;i++)
      for(int ii=0;ii<SFX_NUM;ii++)
        toPlay[i][ii]=0;

    /* DIGI_DIRECTX(1) means I'm expecting directX sound support
       if that isn't present, we then do an auto search, then give up
       on midi input, then give up on sound altogether.  Hopefully,
       the outermost conditional returns 0 errors so the inner loops
       get skipped.

       Update: MIDIs are no longer used in this game, but you can see
       how to make them work in the previous builds (6 and before).
        */
    if(install_sound(DIGI_DIRECTX(1),MIDI_NONE,"data//horiz.wav")!=0)
      {
      if(install_sound(DIGI_AUTODETECT,MIDI_NONE,"data//horiz.wav")!=0)
        {
        if(install_sound(DIGI_AUTODETECT,MIDI_NONE,"data//horiz.wav")!=0)
          {
          maxVolumeMusic=0;
          maxVolumeSFX=0;
          }
        }
      }
    }

  void sound_typ::loadSfxAndMusic()
    {
    pSFX[SFXmenuClick]=load_sample("data//menuclick.wav");
    pSFX[SFXnapalm]=load_sample("data//napalm.wav");
    pSFX[SFXzap]=load_sample("data//zap.wav");
    pSFX[SFXcrash]=load_sample("data//crash.wav");
    pSFX[SFXmajorZap]=load_sample("data//majorzap.wav");
    pSFX[SFXrocketOut]=load_sample("data//rocketout.wav");
    pSFX[SFXelectricDeath]=load_sample("data//electricdeath.wav");
    pSFX[SFXrattleOut]=load_sample("data//rattleout.wav");
    pSFX[SFXbadBuzzer]=load_sample("data//badbuzzer.wav");
    pSFX[SFXdialUp]=load_sample("data//dialup.wav");
    pSFX[SFXengineChug]=load_sample("data//engineChug.wav");
    pSFX[SFXwhoosh]=load_sample("data//whoosh.wav");
    pSFX[SFXpurchase]=load_sample("data//purchase.wav");
    pSFX[SFXempty]=load_sample("data//empty.wav");
    pSFX[SFXbomb]=load_sample("data//bomb.wav");
    pSFX[SFXengUp]=load_sample("data//engup.wav");
    pSFX[SFXengDown]=load_sample("data//engdown.wav");
    pSFX[SFXswish]=load_sample("data//swish.wav");
    pSFX[SFXlightning1]=load_sample("data//lgt1.wav");
    pSFX[SFXlightning2]=load_sample("data//lgt2.wav");
    pSFX[SFXlightning3]=load_sample("data//lgt3.wav");
    pSFX[SFXlightning4]=load_sample("data//lgt4.wav");
    pSFX[SFXplatFall]=load_sample("data//platfall.wav");
    pSFX[SFXplatBoom]=load_sample("data//platboom.wav");

    // loadLevelSong(TITLE_SCREEN_SONG_DIFFICULTY_NUMBER);
    // loaded at menus and stages
    }

  void sound_typ::loopSong(char *fileName)
    {
    if(maxVolumeMusic<1)
      maxVolumeMusic=1; // no zero or neg, or music won't play
    if(LEVEL_SONG!=NULL)
      destroy_sample(LEVEL_SONG);
    LEVEL_SONG=load_sample(fileName);
    play_sample(LEVEL_SONG, maxVolumeMusic, 500, 1000, TRUE);
    adjust_sample(LEVEL_SONG, maxVolumeMusic, 127, 1000, 1);
    }

  void sound_typ::loadLevelSong(int songIndex)
    {
    if(!maxVolumeMusic)
      return;
    if(songIndex==TITLE_SCREEN_SONG_DIFFICULTY_NUMBER) // title screen
      loopSong("data//mus.wav");
    else if(songIndex==CORP_MOMPOP)
      loopSong("data//mus2.wav");
    else if(songIndex==CORP_PIRATES)
      loopSong("data//mus4.wav");
    else if(songIndex==CORP_BIGGOV)
      loopSong("data//mus3.wav");
    else if(songIndex==CORP_HOME)
      loopSong("data//mus1.wav");
    else if(songIndex==CORP_BUG)
      loopSong("data//mus5.wav");
    }

  void sound_typ::unloadSfxAndMusic()
    {
    for(int i=0;i<SFX_NUM;i++)
      destroy_sample(pSFX[i]);
    destroy_sample(LEVEL_SONG);
    }

  void sound_typ::queueSound(int SFXindex,int SFXvolume,float SFXspot_x)
    {
    if(SFXvolume>maxVolumeSFX) // just cap volume at this value
      toPlay[SFXPARAM_VOLUME][SFXindex]=maxVolumeSFX;
    else
      toPlay[SFXPARAM_VOLUME][SFXindex]=SFXvolume;

    if(SFXspot_x<viewport.x || SFXspot_x>viewport.x+SCREEN_W)
      toPlay[SFXPARAM_VOLUME][SFXindex]>>=1; // half volume off screen

    toPlay[SFXPARAM_STEREO_SPOT][SFXindex]=((((int)SFXspot_x)-viewport.x)<<8)/SCREEN_W;
      // (pixel_x/SCREEN_W)*255 sets the pixel for stereo from 0-255
    }

  void sound_typ::playSound(int SFXindex,int SFXvolume,float SFXspot_x)
    { // mainly used in menus
    if(SFXvolume>maxVolumeSFX) // just cap volume at this value
      toPlay[SFXPARAM_VOLUME][SFXindex]=maxVolumeSFX;
    else
      toPlay[SFXPARAM_VOLUME][SFXindex]=SFXvolume;

    toPlay[SFXPARAM_STEREO_SPOT][SFXindex]=(((int)SFXspot_x)<<8)/SCREEN_W;
      // (pixel_x/SCREEN_W)*255 sets the pixel for stereo from 0-255

    play_sample(pSFX[SFXindex],
         toPlay[SFXPARAM_VOLUME][SFXindex], // 0-255 volume
         toPlay[SFXPARAM_STEREO_SPOT][SFXindex], // 0-255 screen position
         1000, // pitch, relative to 1000 default
         FALSE); // loop state (off)
    toPlay[SFXPARAM_VOLUME][SFXindex]=0; // we've played it, so 0 it
    }

  void sound_typ::playQueuedSounds()
    {
    for(int i=0;i<SFX_NUM;i++)
      {
      if(toPlay[SFXPARAM_VOLUME][i]) // have we requested to play this?
        {
        play_sample(pSFX[i],
           toPlay[SFXPARAM_VOLUME][i], // 0-255 volume
           toPlay[SFXPARAM_STEREO_SPOT][i], // 0-255 screen position
           1000, // pitch, relative to 1000 default
           FALSE); // loop state (off)
        toPlay[SFXPARAM_VOLUME][i]=0; // we've played it, so 0 it
        }
      }
    }



